@@ -20,5 +20,8 @@ class TestApacheConfigBuilder(TestCase):
         self.builder = None
 
     def testCreateConfigContent(self):
-        self.failUnlessEqual(self.builder.createConfigContent(), "")
+        self.assertTrue(self.builder.createConfigContent())
+
+    def testCreateWsgiContent(self):
+        self.assertTrue(self.builder.createWsgiScriptContent())
 
