@@ -6,6 +6,7 @@ import dm.environment
 from dm.cli.base import CommandLineUtility
 from dm.exceptions import KforgeError, MissingConfigurationPath
 import traceback
+from dm.dictionarydef import *
 
 class MethodError(object): pass
 
@@ -14,10 +15,15 @@ class AdministrationUtility(CommandLineUtility):
     
     def __init__(self):
         CommandLineUtility.__init__(self) # old style class
+        self.dictionary = None
         self.appInstance = None
         self.interactive = 0
         self.prompt = '> '
 
+    def getLogger(self):
+        import dm.log
+        return dm.log.log
+
     def default(self, line):
         print 'Error: Not a valid command: %s' % line
         errorStatus = int(not self.interactive)
@@ -37,37 +43,107 @@ class AdministrationUtility(CommandLineUtility):
                 print 'Use "quit" or Ctrl-D (i.e. EOF) to exit.'
                 print ''
 
-    def buildApplication(self):
-        import dm.soleInstance
-        self.appInstance = dm.soleInstance.application
-
     def getApplication(self):
         # Todo: Rename 'appInstance' attribute as 'application'.
         if self.appInstance == None:
             self.buildApplication()
         return self.appInstance
         
+    def buildApplication(self):
+        import dm.soleInstance
+        self.appInstance = dm.soleInstance.application
+
+    def loadAllPlugins(self):
+        for plugin in self.getApplication().registry.plugins:
+            plugin.getSystem()
+
     def convertLineToArgs(self, line):
         unstrippedArgs = line.strip().split()
         self.args = [arg.strip() for arg in unstrippedArgs]
         return self.args
 
+    def do_settings(self, line=None):
+        settings = self.getSettings()
+        for setting in settings:
+            print setting[0], "=", setting[1]
+
+    def getSettings(self):
+        #settings = self.getSystemDictionary().items()
+        self.loadAllPlugins()
+        settings = self.getApplication().dictionary.items()
+        settings.sort()
+        return settings
+
+    def help_settings(self, line=None):
+        usage = \
+'''settings
+
+List system dictionary settings, which are based on KForge configuration file.
+'''
+        print usage
+
+    def do_checksettings(self, line=None):
+        dictionary = self.getApplication().dictionary
+        settings = self.getSettings()
+        errors = []
+        for s in settings:
+            key = s[0]
+            value = s[1]
+            if isinstance(key, DictionaryDef):
+                try:
+                    key.check(value, dictionary)
+                except DictionaryDefinitionError, inst:
+                    print inst
+                    errors.append(inst)
+                else:
+                    print "OK %s: %s" % (key, value)
+        if errors:
+            print
+            print "There were errors."
+            print
+            for inst in errors:
+                print inst
+            print
+            return 1
+        else:
+            print
+            print "Everything looks OK."
+            return 0
+
+    def help_checksettings(self):
+        usage = \
+'''checksettings
+
+Check the configuration options are okay.
+'''
+        print usage
+
     def do_setup(self, line=None):
+        self.convertLineToArgs(line)
+        # Todo: Setup from dump file (optional argument on the command line?).
+        # Todo: Setup with admin password (optional argument on the command line?).
         self.createFilesystem()
+        dumpPath = None
+        if len(self.args) == 1:
+            dumpPath = self.args[0]
         try:
             self.takeDatabaseAction('create')
         except Exception, inst:
             print "Warning: Couldn't create the database: %s" % inst
             print "Trying to init the database anyway..."
-        self.takeDatabaseAction('init')
-        self.buildApplication()
+        if dumpPath:
+            self.migrateLoad(dumpPath)
+        else:
+            self.takeDatabaseAction('init')
+        self.getApplication()
         self.buildApacheConfig()
 
     def help_setup(self, line=None):
         usage = \
-'''setup
+'''setup [DUMPFILE]
 
-Create service from configuration (files, database, Apache config).
+Set up a new KForge service from given KForge configuration file, optionally
+using a DUMPFILE containing domain model data dumped from an existing service.
 '''
         print usage
 
@@ -77,10 +153,11 @@ Create service from configuration (files, database, Apache config).
             print 'Error: Inappropriate arguments\n'
             self.help_migratedump(line)
             return 1
-        dumpFilePath = self.args[0]
+        dumpPath = self.args[0]
+        self.migrateDump(dumpPath)
 
     def migrateDump(self, dumpPath):
-        self.buildApplication()
+        self.getApplication()
         from dm.migrate import DomainModelDumper
         dataDumper = DomainModelDumper()
         dataDumper.dumpDataToFile(dumpPath)
@@ -99,7 +176,7 @@ Dump service object model (equals 'db dump')
             print 'Error: Inappropriate arguments\n'
             self.help_migratedumpfiles(line)
             return 1
-        self.buildApplication()
+        self.getApplication()
         self.migrateDumpFiles()
 
     def migrateDumpFiles(self):
@@ -129,7 +206,7 @@ Dump service filesystem files
         self.migrateLoad(dumpPath)
 
     def migrateLoad(self, dumpPath):
-        self.buildApplication()
+        self.getApplication()
         dataLoader = self.getDomainModelLoaderClass()()
         dataLoader.loadDataFromFile(dumpPath)
         self.touchMigratedDomainModel()
@@ -183,6 +260,9 @@ Load migrated service data (equals 'db init')
                 try:
                     self.takeDatabaseAction(actionName)
                 except Exception, inst:
+                    trace = traceback.format_exc()
+                    print trace
+                    # Fix up the logging, and write the traceback to the log.
                     print "Error: Couldn't %s the domain data: %s" % (actionName, str(inst))
                     return 1
             else:
@@ -195,7 +275,7 @@ Load migrated service data (equals 'db init')
             return 1
        
     def takeDatabaseAction(self, actionName):
-        dictionary = self.constructSystemDictionary() # Need to have DJANGO_SETTING_MODULE set.
+        dictionary = self.getSystemDictionary() # Need to have DJANGO_SETTING_MODULE set.
         dbUtilClass = self.getDatabaseUtilityClass()
         dbUtil = dbUtilClass() # Todo: Pass in the dictionary.
         actionMethod = getattr(dbUtil, actionName)
@@ -281,10 +361,18 @@ service configuration file (see 'templates_dir' and 'media_dir').
 
     def createFilesystem(self):
         from dm.command.initialise import InitialiseFilesystem
-        dictionary = self.constructSystemDictionary()
+        dictionary = self.getSystemDictionary()
         command = InitialiseFilesystem(dictionary)
         command.execute()
 
+    def getSystemDictionary(self):
+        if self.dictionary == None:
+            if self.appInstance:
+                self.dictionary = self.appInstance.dictionary
+            else:
+                self.dictionary = self.constructSystemDictionary()
+        return self.dictionary
+
     def constructSystemDictionary(self):
         from dm.dictionary import SystemDictionary
         return SystemDictionary()
@@ -295,7 +383,7 @@ service configuration file (see 'templates_dir' and 'media_dir').
             self.help_apacheconfig(line)
             return 1
         try:
-            self.buildApplication()
+            self.getApplication()
         except Exception, inst:
             msg = "Error: %s" % str(inst)
             print msg
