@@ -5,11 +5,11 @@ class DomainConfigMaker(object):
 
     template = ''
 
-    usage  = '''usage: %prog [path]
+    usage  = '''usage: %prog [OPTIONS] [PATH]
 
-Generate new configuration file from template, see --help for options.
+Generate new configuration file, see --help for options.
 
-Writes file to path if given, otherwise prints to stdout.
+Writes file to PATH if given, otherwise prints to stdout.
         '''
 
     def __init__(self, systemName):
@@ -25,7 +25,11 @@ Writes file to path if given, otherwise prints to stdout.
             parser.print_help()
             sys.exit(1)
         elif args:
-            configPath = args[0]
+            configPath = os.path.abspath(args[0])
+
+            if os.path.exists(configPath):
+                print 'Error: File already exists on path: %s' % configPath
+                sys.exit(1)
         else:
             configPath = None
         # Define configuration argument.
@@ -35,9 +39,21 @@ Writes file to path if given, otherwise prints to stdout.
         configLines = self.makeLines(optionLines)
         # Output configuration lines.
         if configPath:
-            configFile = file(configPath, 'w')
-            configFile.writelines(configLines)
-            configFile.close()
+            # Not readable or writable or executable for other, and not
+            # writable by group.
+            oldumask = os.umask(0o027)
+            try:
+                configDirPath = os.path.dirname(configPath)
+                if not os.path.exists(configDirPath):
+                    os.makedirs(configDirPath)
+                if os.path.exists(configPath):
+                    msg = 'Error: File already exists on path: %s' % configPath
+                    raise Exception, msg 
+                configFile = file(configPath, 'w')
+                configFile.writelines(configLines)
+                configFile.close()
+            finally:
+                os.umask(oldumask)
         else:
             print "".join(configLines)
 
@@ -48,15 +64,25 @@ Writes file to path if given, otherwise prints to stdout.
             #default='/path/to/%s' % self.systemName,
             help="""The path to the folder for service files.""")
 
+        parser.add_option(
+            '--site-host',
+            dest='siteHost',
+            help="""The site host name where the KForge service will be provided (default: projects.DNSDOMAINNAME).""")
+
+        parser.add_option(
+            '--dns-domain-name',
+            dest='dnsDomainName',
+            help="""The registered domain name within which the KForge service will be available (default: result of 'dnsdomainname' command).""")
+
         parser.add_option(
             '--service-name',
             dest='serviceName',
             help='The service name to be used.')
 
         parser.add_option(
-            '--system-mode',
-            dest='systemMode',
-            help='The mode of the system (production or development).')
+            '--umask',
+            dest='umask',
+            help='The umask for new files (default: 0o007).')
 
         parser.add_option(
             '--password-digest-secret',
@@ -64,14 +90,27 @@ Writes file to path if given, otherwise prints to stdout.
             help='The "secret" used as "salt" for password digests. Use the old system value when upgrading.')
 
         parser.add_option(
-            '--environment-timezone',
-            dest='environmentTimezone',
-            help='The timezone to be used.')
+            '--enable-reload-apache',
+            dest='enableReloadApache',
+            action='store_true',
+            help='Enable auto-reloading of the auto-generated Apache config file.')
 
         parser.add_option(
-            '--domain-name',
-            dest='domainName',
-            help="""The fully qualified domain name where the site will be served.""")
+            '--enable-memoization',
+            dest='enableMemoization',
+            action='store_true', 
+            help='Enable memoization in the access controller.')
+
+        parser.add_option(
+            '--enable-model-cache',
+            dest='enableModelCache',
+            action='store_true', 
+            help='Enable cacheing in the domain object model.')
+
+        parser.add_option(
+            '--timezone',
+            dest='timezone',
+            help='The timezone to be used.')
 
         parser.add_option(
             '--db-type',
@@ -82,7 +121,7 @@ Writes file to path if given, otherwise prints to stdout.
         parser.add_option(
             '--db-name',
             dest='dbName',
-            help='The name of the database instance.')
+            help='The name of the database instance (or path to SQLite file).')
 
         parser.add_option(
             '--db-user',
@@ -114,12 +153,6 @@ Writes file to path if given, otherwise prints to stdout.
             dest='virtualenvBinDir',
             help='The virtual environment scripts folder.')
 
-        parser.add_option(
-            '--enable-memoization',
-            dest='enableMemoization',
-            action='store_true', 
-            help='Enable memoization in the access controller.')
-
         parser.add_option(
             '--log-file',
             dest='logFile',
@@ -165,6 +198,11 @@ Writes file to path if given, otherwise prints to stdout.
             dest='emailSmtpHost',
             help="""The host to which SMTP connections will be made when sending email""")
     
+        parser.add_option(
+            '--system-mode',
+            dest='systemMode',
+            help='The mode of the system (production or development).')
+
     def addOptionLines(self, options, optionLines):
         optionLines.append('[DEFAULT]')
         if options.masterDir:
@@ -174,28 +212,32 @@ Writes file to path if given, otherwise prints to stdout.
             optionLines.append('service_name = %s' % options.serviceName)
         if options.systemMode:
             optionLines.append('system_mode = %s' % options.systemMode)
-        if options.domainName:
-            optionLines.append('domain_name = %s' % options.domainName)
+        if options.umask:
+            optionLines.append('umask = %s' % options.umask)
+        if options.dnsDomainName:
+            optionLines.append('dns_domain_name = %s' % options.dnsDomainName)
+        if options.siteHost:
+            optionLines.append('site_host = %s' % options.siteHost)
         optionLines.append('password_digest_secret = %s' % (options.passwordDigestSecret or self.generateNewSecretKey()))
-        if options.environmentTimezone:
+        if options.timezone:
             optionLines.append('[environment]')
-            optionLines.append('timezone = %s' % options.environmentTimezone)
+            optionLines.append('timezone = %s' % options.timezone)
+        virtualenvBinDir = None
         if options.virtualenvBinDir:
             optionLines.append('[virtualenv]')
-            optionLines.append('bin_dir = %s' % options.virtualenvBinDir)
+            virtualenvBinDir = os.path.abspath(options.virtualenvBinDir)
+        else:
+            pythonBinDir = os.path.dirname(sys.executable)
+            if os.path.exists(os.path.join(pythonBinDir, 'activate')):
+                virtualenvBinDir = pythonBinDir
+        if virtualenvBinDir:
+            optionLines.append('[virtualenv]')
+            optionLines.append('bin_dir = %s' % virtualenvBinDir)
         optionLines.append('[db]')
         if not options.dbType:
-            if options.dbName or options.dbPass or options.dbHost:
+            if options.dbPass or options.dbHost:
                 options.dbType = 'postgres'
-            # Don't set this, so that the value remain uncommented in the config file.
-            #else:
-            #    options.dbType = 'sqlite'
-        if options.dbType in ['sqlite', None]:
-            if not options.dbName:
-                # Don't set this, so that the value remain uncommented in the config file.
-                #options.dbName = '%(master_dir)s/var/sqlite.db'
-                pass
-        elif options.dbType in ['mysql', 'postgres']:
+        if options.dbType in ['mysql', 'postgres']:
             msg = "Error: No database %%s (required for %s databases)." % options.dbType
             if not options.dbName:
                 print msg % 'name'
@@ -224,6 +266,9 @@ Writes file to path if given, otherwise prints to stdout.
         if options.enableMemoization and options.systemMode != 'development':
             optionLines.append('[memos]')
             optionLines.append('enabled = on')
+        if options.enableModelCache and options.systemMode != 'development':
+            optionLines.append('[model_cache]')
+            optionLines.append('enable = on')
         optionLines.append('[django]')
         optionLines.append('secret_key = %s' % self.generateNewSecretKey())
         if options.templatesDir:
@@ -231,8 +276,10 @@ Writes file to path if given, otherwise prints to stdout.
         if options.logFile:
             optionLines.append('[logging]')
             optionLines.append('log_file = %s' % options.logFile)
-        if options.apacheConfigFile or options.mediaDir or options.wsgiFile:
+        if options.apacheConfigFile or options.mediaDir or options.wsgiFile or options.enableReloadApache:
             optionLines.append('[www]')
+            if options.enableReloadApache:
+                optionLines.append('enable_reload_apache = on')
             if options.apacheConfigFile:
                 optionLines.append('apache_config_file = %s' % options.apacheConfigFile)
             if options.mediaDir:
