@@ -52,7 +52,7 @@ class GrantAccess(AccessControlCommand):
             permission = protectionObject.permissions[self.action]
             if not permission in self.role.grants:
                 self.role.grants.create(permission)
-        except StandardException, inst:
+        except Exception, inst:
             error = "Could not grant permission on role '%s' to '%s' object '%s': %s" % (self.role.name, self.action.name, self.protectedObject, inst)
             self.raiseError(error)
 
@@ -85,22 +85,18 @@ class GrantStandardSystemAccess(Command):
         self.createGrants()
 
     def createGrants(self):
-
         # Administrator allowed everything.
-        
         administrator = self.registry.roles['Administrator']
         for permission in self.protectionObject.permissions:
             if not permission in administrator.grants:
                 administrator.grants.create(permission)
 
         # Developer allowed to read.
-        
         developer = self.registry.roles['Developer']
         if not self.read in developer.grants:
             developer.grants.create(self.read)
 
         # Friend allowed to read.
-        
         friend = self.registry.roles['Friend']
         if not self.read in friend.grants:
             friend.grants.create(self.read)
@@ -132,7 +128,6 @@ class GrantStandardDevelopmentAccess(GrantStandardSystemAccess):
         super(GrantStandardDevelopmentAccess, self).createGrants()
         
         # Developer allowed to update.
-        
         developer = self.registry.roles['Developer']
         if not self.update in developer.grants:
             developer.grants.create(self.update)
