@@ -5,26 +5,33 @@ from dm.command.accesscontrol import GrantStandardSystemAccess
 from dm.command.person import *
 import shutil
 import pkg_resources
-from dm.dictionarywords import TEMPLATES_RESOURCE_PATH, MEDIA_RESOURCE_PATH
-from dm.dictionarywords import SYSTEM_NAME, SYSTEM_PACKAGE_NAME, DJANGO_TEMPLATES_DIR, MEDIA_PATH
+from dm.dictionarywords import *
 
 class InitialiseDomainModelBase(Command):
 
     def execute(self):
         super(InitialiseDomainModelBase, self).execute()
+        self.failIfDomainModelIsInitialised()
+        self.initialiseDomainModelObjects()
+
+    def failIfDomainModelIsInitialised(self):
+        if self.registry.states.count():
+            raise Exception, "Database is already initialised."
+
+    def initialiseDomainModelObjects(self):
         self.createStates()
         self.createSystem()
         self.createActions()
 
     def createStates(self):
-        self.registry.states.create('pending')
         self.registry.states.create('active')
         self.registry.states.create('deleted')
+        self.registry.states.create('pending')
         
     def createSystem(self):
         self.registry.systems.create(
-            version=self.dictionary['system_version'],
-            mode=self.dictionary['system_mode'],
+            version=self.dictionary[SYSTEM_VERSION],
+            mode=self.dictionary[SYSTEM_MODE],
         )
 
     def createActions(self):
@@ -50,9 +57,9 @@ class InitialiseDomainModel(InitialiseDomainModelBase):
         self.createProtectionObjects()
         self.createGrants()
         self.createRefusals()
-        self.createPersons()
         self.createAccessControlPlugin()
-        if self.dictionary['system_mode'] == 'development':
+        self.createPersons()
+        if self.dictionary[SYSTEM_MODE] == 'development':
             self.createTestPlugins()
             self.setUpTestFixtures()
         self.commitSuccess()
@@ -69,6 +76,7 @@ class InitialiseDomainModel(InitialiseDomainModelBase):
         self.registry.protectionObjects.create('System')
         self.registry.protectionObjects.create('Person')
         self.registry.protectionObjects.create('Plugin')
+        self.registry.protectionObjects.create('Role')
 
     def createGrants(self):
         self.grantAdministratorAccess()
@@ -100,20 +108,20 @@ class InitialiseDomainModel(InitialiseDomainModelBase):
         pass
 
     def createPersons(self):
-        domainName = self.dictionary['domain_name']
+        domainName = self.dictionary[SITE_HOST]
         cmd = PersonCreate('admin', 
             role=self.adminRole,
             fullname='Administrator',
-            email='kforge-admin@%s' % domainName
         )
         cmd.execute()
         self.adminPerson = cmd.person
         self.adminPerson.setPassword('pass')
         self.adminPerson.save()
+        self.adminPerson.setEmailAddress('kforge-admin@%s' % domainName)
         
-        visitorRoleName = self.dictionary['visitor_role']
+        visitorRoleName = self.dictionary[VISITOR_ROLE]
         visitorRole = self.registry.roles[visitorRoleName]
-        cmd = PersonCreate(self.dictionary['visitor'],
+        cmd = PersonCreate(self.dictionary[VISITOR_NAME],
             role = visitorRole,
             fullname='Visitor',
         )
@@ -129,44 +137,44 @@ class InitialiseDomainModel(InitialiseDomainModelBase):
         plugins.create('example')
     
     def setUpTestFixtures(self):
-        domainName = self.dictionary['domain_name']
-        personRoleName = self.dictionary['person_role']
+        domainName = self.dictionary[SITE_HOST]
+        personRoleName = self.dictionary[INITIAL_PERSON_ROLE]
         personRole = self.registry.roles[personRoleName]
         # do not reuse roles set in other methods as this method
         # should be callable on its own
-        adminPerson = self.registry.persons['admin']
+        adminPerson = self.registry.people['admin']
         adminRole = self.registry.roles['Administrator']
         friendRole = self.registry.roles['Friend']
 
         cmd = PersonCreate('levin',
             role = personRole,
             fullname=u'Levin \xf3',
-            email='levin@%s' % domainName
         )
         cmd.execute()
         levin = cmd.person
         levin.setPassword('levin')
         levin.save()
+        levin.setEmailAddress('levin@%s' % domainName)
         
         cmd = PersonCreate('natasha',
             role = personRole,
             fullname=u'Natasha \xf3',
-            email='natasha@%s' % domainName
         )
         cmd.execute()
         natasha = cmd.person
         natasha.setPassword('natasha')
         natasha.save()
-        visitor = self.registry.persons[self.dictionary['visitor']]
+        natasha.setEmailAddress('natasha@%s' % domainName)
+        visitor = self.registry.people[self.dictionary[VISITOR_NAME]]
         
     def tearDownTestFixtures(self):
         # [[TODO: factor this out into a command class]]
         
         def purgePerson(personName):
-            if self.registry.persons.has_key(personName):
-                self.registry.persons[personName].delete()
-            if self.registry.persons.getAll().has_key(personName):
-                self.registry.persons.getAll()[personName].purge()
+            if self.registry.people.has_key(personName):
+                self.registry.people[personName].delete()
+            if self.registry.people.getAll().has_key(personName):
+                self.registry.people.getAll()[personName].purge()
         purgePerson('natasha')
         purgePerson('levin')
         purgePerson('anna')
@@ -184,47 +192,59 @@ class InitialiseFilesystem(object):
         self.verbose = verbose
 
     def execute(self):
-        self.installTemplates()
-        self.installMedia()
+        umask = os.umask(0o027)
+        try:
+            varDirPath = os.path.join(self.dictionary[FILESYSTEM_PATH], 'var')
+            logDirPath = os.path.dirname(self.dictionary[LOG_PATH])
+            if not os.path.exists(varDirPath):
+                os.makedirs(varDirPath)
+            if not os.path.exists(logDirPath):
+                os.makedirs(logDirPath)
+            os.chmod(logDirPath, 0o770)
+            if not os.path.exists(self.dictionary[IMAGES_DIR_PATH]):
+                os.makedirs(self.dictionary[IMAGES_DIR_PATH])
+            self.installTemplates()
+            self.installMedia()
+        finally:
+            os.umask(umask)
 
     def installTemplates(self):
+        #self.installDmPackageTemplates()
+        self.installApplicationTemplates()
+
+    def installDmPackageTemplates(self):
+        if self.packageName != 'dm':
+            installPath = self.dictionary[DJANGO_TEMPLATES_DIR]
+            self.installFiles('templates', self.templatesResourcePath, installPath, packageName='dm')
+
+    def installApplicationTemplates(self):
         installPath = self.dictionary[DJANGO_TEMPLATES_DIR]
         self.installFiles('templates', self.templatesResourcePath, installPath)
 
     def installMedia(self):
         installPath = self.dictionary[MEDIA_PATH]
+        if os.path.exists(installPath):
+            raise Exception, "Path already exists: %s" % installPath
         self.installFiles('media', self.mediaResourcePath, installPath)
 
-    def installFiles(self, purpose, resourcePath, installPath):
+    def installFiles(self, purpose, resourcePath, installPath, packageName=None):
         if not installPath:
             raise Exception, "installPath is missing."
         installPath = os.path.normpath(installPath)
-        try:
-            packageModule = __import__(self.packageName, '', '', '*')
-            packagePath = os.path.dirname(packageModule.__file__)
-            fullResourcePath = os.path.join(packagePath, resourcePath)
-        except:
-            # Hopefully this isn't necessary anymore. :-)
-            try:
-                fullResourcePath = pkg_resources.resource_filename(
-                    pkg_resources.Requirement.parse(self.systemName),
-                    '%s/%s' % (self.packageName, resourcePath )
-                )
-            except Exception, inst:
-                msg = "Couldn't make full '%s' package resource path for '%s': %s" % (
-                    self.packageName,
-                    resourcePath,
-                    repr(inst)
-                )
-                raise Exception, msg
         if os.path.exists(installPath):
-            print "Skipping %s: The folder already exists: %s" % (purpose, installPath)
-        else:
-            self.assertFolder(os.path.dirname(installPath))
+            raise Exception, "Folder for '%s' already exists : %s" % (purpose, installPath)
+        if not packageName:
+            packageName = self.packageName
+        packageModule = __import__(packageName, '', '', '*')
+        packagePath = os.path.dirname(packageModule.__file__)
+        fullResourcePath = os.path.join(packagePath, resourcePath)
+        if not os.path.exists(os.path.dirname(installPath)):
+            os.makedirs(installPath)
+        try:
             shutil.copytree(fullResourcePath, installPath)
-
-    def assertFolder(self, folderPath):
-        if not os.path.exists(folderPath):
-            os.makedirs(folderPath)
-
+        except OSError, inst:
+            msg = "Couldn't install files from package resources: %s to %s: %s" % (
+                fullResourcePath, installPath, inst
+            )
+            raise Exception, msg
 
