@@ -10,7 +10,7 @@ class PersonCommand(Command):
         super(PersonCommand, self).__init__(name=name)
         self.name = name
         self.person = None
-        self.persons = self.registry.persons
+        self.people = self.registry.people
 
 
 class PersonCreate(DomainObjectCreate):
@@ -50,10 +50,10 @@ class PersonCreate(DomainObjectCreate):
 #        if self.reservedNames.match(self.name):
 #            message = "Name '%s' is reserved." % self.name
 #            self.raiseError(message)
-#        if self.name in self.persons.getAll():
+#        if self.name in self.people.getAll():
 #            message = "A person already exists with name '%s'." % self.name
 #            self.raiseError(message)
-#        self.person = self.persons.create(self.name)
+#        self.person = self.people.create(self.name)
 #        self.commitSuccess()
 
 
@@ -66,10 +66,10 @@ class PersonDelete(PersonCommand):
     def execute(self):
         "Make it so."
         super(PersonDelete, self).execute()
-        if not self.name in self.persons:
+        if not self.name in self.people:
             message = "No person found with name '%s'." % self.name
             self.raiseError(message)
-        self.person = self.persons[self.name]
+        self.person = self.people[self.name]
         self.person.delete()
         self.commitSuccess()
 
@@ -83,10 +83,10 @@ class PersonUndelete(PersonCommand):
     def execute(self):
         "Make it so."
         super(PersonUndelete, self).execute()
-        if not self.name in self.persons.getDeleted():
+        if not self.name in self.people.getDeleted():
             message = "No deleted person found with name '%s'." % self.name
             self.raiseError(message)
-        person = self.persons.getDeleted()[self.name]
+        person = self.people.getDeleted()[self.name]
         person.undelete()
         self.commitSuccess()
 
@@ -100,10 +100,10 @@ class PersonPurge(PersonCommand):
     def execute(self):
         "Make it so."
         super(PersonPurge, self).execute()
-        if not self.name in self.persons.getDeleted():
+        if not self.name in self.people.getDeleted():
             message = "No deleted person found with name '%s'." % self.name
             self.raiseError(message)
-        person = self.persons.getDeleted()[self.name]
+        person = self.people.getDeleted()[self.name]
         person.purge()
         self.commitSuccess()
 
@@ -118,7 +118,7 @@ class PersonRead(PersonCommand):
         "Make it so."
         super(PersonRead, self).execute()
         try:
-            person = self.persons[self.name]
+            person = self.people[self.name]
             self.person = person
         except KforgeRegistryKeyError, inst:
             #message = "No person found with name '%s'." % self.name
@@ -151,32 +151,36 @@ class AllPersonRead(PersonRead):
 
     def __init__(self, name):
         super(AllPersonRead, self).__init__(name)
-        self.persons = self.persons.getAll()
+        self.people = self.people.getAll()
 
 
 class PersonList(PersonCommand):
-    "Command to list registered persons."
+    "Command to list registered people."
 
-    def __init__(self, userQuery='', startsWith='', startsWithAttributeName=''):
+    def __init__(self, userQuery='', startsWith='', startsWithAttributeName='', viewer=None):
         super(PersonList, self).__init__('')
         self.userQuery = userQuery
         self.startsWith = startsWith
         self.startsWithAttributeName = startsWithAttributeName
+        self.viewer = viewer
 
     def execute(self):
         "Make it so."
         super(PersonList, self).execute()
+        kwds = {}
+        viewerName = self.viewer.name if self.viewer else ''
+        kwds['__accessedBy__'] = viewerName
         if self.startsWith:
-            selection = self.persons.startsWith(
+            selection = self.people.startsWith(
                 value=self.startsWith,
-                attributeName=self.startsWithAttributeName
-            )
+                attributeName=self.startsWithAttributeName,
+                **kwds)
             self.results = [p for p in selection]
         elif self.userQuery:
-            selection = self.persons.search(
-                userQuery=self.userQuery
-            )
+            selection = self.people.search(
+                userQuery=self.userQuery, **kwds)
             self.results = [p for p in selection]
         else:
-            self.results = [p for p in self.persons]
+            self.results = self.people.findDomainObjects(**kwds)
+        return self.results
 
