@@ -27,16 +27,16 @@ class TestPersonCreate(TestCase):
 
     def tearDown(self):
         try:
-            person = self.registry.persons.getAll()[self.fixtureName]
+            person = self.registry.people.getAll()[self.fixtureName]
             person.delete()
             person.purge()
         except:
             pass
 
     def testExecute(self):
-        self.failIf(self.fixtureName in self.registry.persons)
+        self.failIf(self.fixtureName in self.registry.people)
         self.command.execute()
-        self.failUnless(self.fixtureName in self.registry.persons)
+        self.failUnless(self.fixtureName in self.registry.people)
 
     def testErrorPersonExists(self):
         self.command.execute()
@@ -50,9 +50,9 @@ class TestPersonDelete(TestCase):
         super(TestPersonDelete, self).setUp()
         self.fixtureName = 'TestPersonDelete'
         try:
-            self.person = self.registry.persons.create(self.fixtureName)
+            self.person = self.registry.people.create(self.fixtureName)
         except:
-            person = self.registry.persons.getAll()[self.fixtureName]
+            person = self.registry.people.getAll()[self.fixtureName]
             person.delete()
             person.purge()
             raise
@@ -60,16 +60,16 @@ class TestPersonDelete(TestCase):
 
     def tearDown(self):
         try:
-            person = self.registry.persons.getAll()[self.fixtureName]
+            person = self.registry.people.getAll()[self.fixtureName]
             person.delete()
             person.purge()
         except:
             pass
 
     def testExecute(self):
-        self.failUnless(self.fixtureName in self.registry.persons)
+        self.failUnless(self.fixtureName in self.registry.people)
         self.command.execute()
-        self.failIf(self.fixtureName in self.registry.persons)
+        self.failIf(self.fixtureName in self.registry.people)
 
     def testErrorNoPerson(self):
         self.command.execute()
@@ -83,10 +83,10 @@ class TestPersonUndelete(TestCase):
         super(TestPersonUndelete, self).setUp()
         self.fixtureName = 'TestPersonUndelete'
         try:
-            self.person = self.registry.persons.create(self.fixtureName)
+            self.person = self.registry.people.create(self.fixtureName)
             self.person.delete()
         except:
-            person = self.registry.persons.getAll()[self.fixtureName]
+            person = self.registry.people.getAll()[self.fixtureName]
             person.delete()
             person.purge()
             raise
@@ -94,7 +94,7 @@ class TestPersonUndelete(TestCase):
 
     def tearDown(self):
         try:
-            person = self.registry.persons.getAll()[self.fixtureName]
+            person = self.registry.people.getAll()[self.fixtureName]
             person.delete()
             person.purge()
         except:
@@ -102,10 +102,10 @@ class TestPersonUndelete(TestCase):
         self.command = None
 
     def testExecute(self):
-        self.failUnless(self.fixtureName in self.registry.persons.getDeleted())
+        self.failUnless(self.fixtureName in self.registry.people.getDeleted())
         self.command.execute()
-        self.failIf(self.fixtureName in self.registry.persons.getDeleted())
-        self.failUnless(self.fixtureName in self.registry.persons)
+        self.failIf(self.fixtureName in self.registry.people.getDeleted())
+        self.failUnless(self.fixtureName in self.registry.people)
 
     def testErrorNoPerson(self):
         self.command.execute()
@@ -119,10 +119,10 @@ class TestPersonPurge(TestCase):
         super(TestPersonPurge, self).setUp()
         self.fixtureName = 'TestPersonPurge'
         try:
-            self.person = self.registry.persons.create(self.fixtureName)
+            self.person = self.registry.people.create(self.fixtureName)
             self.person.delete()
         except:
-            person = self.registry.persons.getAll()[self.fixtureName]
+            person = self.registry.people.getAll()[self.fixtureName]
             person.delete()
             person.purge()
             raise
@@ -130,7 +130,7 @@ class TestPersonPurge(TestCase):
 
     def tearDown(self):
         try:
-            person = self.registry.persons.getAll()[self.fixtureName]
+            person = self.registry.people.getAll()[self.fixtureName]
             person.delete()
             person.purge()
         except:
@@ -138,11 +138,11 @@ class TestPersonPurge(TestCase):
         self.command = None
 
     def testExecute(self):
-        self.failUnless(self.fixtureName in self.registry.persons.getDeleted())
+        self.failUnless(self.fixtureName in self.registry.people.getDeleted())
         self.command.execute()
-        self.failIf(self.fixtureName in self.registry.persons.getDeleted())
-        self.failIf(self.fixtureName in self.registry.persons)
-        self.failIf(self.fixtureName in self.registry.persons.getAll())
+        self.failIf(self.fixtureName in self.registry.people.getDeleted())
+        self.failIf(self.fixtureName in self.registry.people)
+        self.failIf(self.fixtureName in self.registry.people.getAll())
 
     def testErrorNoPerson(self):
         self.command.execute()
@@ -157,9 +157,9 @@ class TestPersonRead(TestCase):
     def setUp(self):
         super(TestPersonRead, self).setUp()
         try:
-            self.person = self.registry.persons.create(self.personName)
+            self.person = self.registry.people.create(self.personName)
         except:
-            person = self.registry.persons.getAll()[self.personName]
+            person = self.registry.people.getAll()[self.personName]
             person.delete()
             person.purge()
             raise
@@ -167,7 +167,7 @@ class TestPersonRead(TestCase):
 
     def tearDown(self):
         try:
-            person = self.registry.persons.getAll()[self.personName]
+            person = self.registry.people.getAll()[self.personName]
             person.delete()
             person.purge()
         except:
@@ -187,16 +187,16 @@ class TestPersonList(TestCase):
     def setUp(self):
         super(TestPersonList, self).setUp()
         try:
-            self.person = self.registry.persons.create(self.personName)
+            self.person = self.registry.people.create(self.personName)
         except:
-            person = self.registry.persons.getAll()[self.personName]
+            person = self.registry.people.getAll()[self.personName]
             person.delete()
             person.purge()
         self.command = dm.command.person.PersonList()
 
     def tearDown(self):
         try:
-            person = self.registry.persons.getAll()[self.personName]
+            person = self.registry.people.getAll()[self.personName]
             person.delete()
             person.purge()
         except:
@@ -204,7 +204,7 @@ class TestPersonList(TestCase):
 
     def testExecute(self):
         self.command.execute()
-        self.command.persons[self.personName]
+        self.command.people[self.personName]
 
 
 class PersonAuthenticateTest(TestCase):
