@@ -1,6 +1,5 @@
 import unittest
 import StringIO
-
 from dm.config import ConfigFileReader
 
 def suite():
@@ -12,8 +11,7 @@ def suite():
 
 class TestConfigFileReader(unittest.TestCase):
     
-    def setUp(self):
-        str1 = """[DEFAULT]
+    configText = """[DEFAULT]
 var_1: var1
 var_2: var2/var1
 # core stuff
@@ -21,9 +19,11 @@ var_2: var2/var1
 var_2 = var2
 var_3: /var3/%(var_1)s
 """
-        configFile1 = StringIO.StringIO(str1)
-        self.configParser = ConfigFileReader()
-        self.configParser.readfp(configFile1)
+
+    def setUp(self):
+        configFile1 = StringIO.StringIO(self.configText)
+        self.reader = ConfigFileReader()
+        self.reader.readfp(configFile1)
         self.expectedDictionary = {
             'var_1'     : 'var1',
             'var_2'     : 'var2/var1',
@@ -35,17 +35,17 @@ var_3: /var3/%(var_1)s
     def test_convertKey(self):
         keyname = 'sect1.optionA'
         exp = ('sect1', 'optionA')
-        out = self.configParser.convertKey(keyname)
+        out = self.reader.convertKey(keyname)
         self.assertEqual(exp, out)
 
     def test___set_item__(self):
         keyname = 'newsection.abc'
         value = 'xyz'
-        self.configParser[keyname] = value
-        self.assertEqual(value, self.configParser[keyname])
+        self.reader[keyname] = value
+        self.assertEqual(value, self.reader[keyname])
 
     def test_as_dictionary(self):
-        config = self.configParser
+        config = self.reader
         for word in self.expectedDictionary.keys():
             configWord = config[word]
             expectedWord = self.expectedDictionary[word]
@@ -56,15 +56,15 @@ var_3: /var3/%(var_1)s
         key2 = 'core.var_2'
         key3 = 'core.xyz'
         key4 = 'nonexistent.blah'
-        self.failUnless(self.configParser.has_key(key1))
-        self.failUnless(self.configParser.has_key(key2))
-        self.failIf(self.configParser.has_key(key3))
-        self.failIf(self.configParser.has_key(key4))
+        self.failUnless(self.reader.has_key(key1))
+        self.failUnless(self.reader.has_key(key2))
+        self.failIf(self.reader.has_key(key3))
+        self.failIf(self.reader.has_key(key4))
 
     def test_keys(self):
-        out = self.configParser.keys()
+        out = self.reader.keys()
         exp = ['var_2', 'var_1', 'core.var_3', 'core.var_2', 'core.var_1' ]
         out.sort()
         exp.sort()
         self.assertEqual(exp, out)
-        
+
