@@ -33,29 +33,29 @@ from dm.exceptions import *
 import sqlobject
 sqoVer = None
 isSqoSupported = False
-sqoVers = ['0.7', '0.8', '0.9', '0.10', '0.11', '0.12', '0.13', '0.14', '0.15', '1.0']
-if sqlobject.__doc__ != None:
+sqoVers = ['0.7', '0.8', '0.9', '0.10', '0.11', '0.12', '0.13', '0.14', '0.15', '1.0', '1.1', '1.2', '1.3']
+if hasattr(sqlobject, 'version'):
+    sqoVerSplit = sqlobject.version.split('.')
+    if len(sqoVerSplit) >= 2:
+        sqoVer = ".".join(sqoVerSplit[0:2])
+elif sqlobject.__doc__ != None:
     sqoDocSplit = sqlobject.__doc__.split(' ')
     if len(sqoDocSplit) > 1:
         sqoVerSplit = sqoDocSplit[1].split('.')
-        if len(sqoVerSplit) > 2:
+        if len(sqoVerSplit) >= 2:
             sqoVer = ".".join(sqoVerSplit[0:2])
+
 if sqoVer in sqoVers:
     isSqoSupported = True
-else:
-    # Debian Etch-ish support (SQLObject 0.7.1).
-    if sqlobject.__doc__ == None and \
-    hasattr(sqlobject.SQLObject, 'sqlmeta'):
-        isSqoSupported = True
-        sqoVer = '0.7'
 if not isSqoSupported:
-    msg = "Imported SQLObject not supported by dm."
+    msg = "Imported SQLObject (%s) not supported by dm." % sqoVer
     msg += "\nImported SQLObject package doc: %s" % sqlobject.__doc__
     msg += "Imported SQLObject package path: %s" % sqlobject.__path__
     msg += "\nNeeds latest version of any of: %s." % ", ".join(sqoVers)
     raise ORMError, msg 
 
 from sqlobject import *
+from sqlobject.sqlbuilder import func
 from sqlobject.converters import sqlrepr
 from dm.ioc import *
 import dm.dom.meta
@@ -64,6 +64,7 @@ from dm.ioc import *
 from dm.dictionarywords import *
 import datetime
 import weakref
+import traceback
 
 debug = RequiredFeature('Debug')
 logger = RequiredFeature('Logger')
@@ -82,20 +83,32 @@ class ConnectionFacade(object):
         dbPass = dictionary[DB_PASS]
         dbHost = dictionary[DB_HOST]
         dbName = dictionary[DB_NAME]
+        DB_NAME.test(dbName, dictionary)
         if dictionary.get(DB_URI, None):
             uri = dictionary[DB_URI]
         elif dbType == 'sqlite':
             uri = "sqlite://%s" % dbName
         else:
             uri = "%s://%s:%s@%s/%s" % (dbType, dbUser, dbPass, dbHost, dbName)
-        logger.info('Database: Connecting to %s.' % uri)
+        uriParams = {}
+        #uriParams['debug'] = '1'
+        uriParams['loglevel'] = dictionary[LOG_LEVEL].lower()
+        uriParams['logger'] = dictionary[SYSTEM_NAME]
+        uriParams['cache'] = '0'
+        if dbType == 'postgres':
+            uriParams['driver'] = 'psycopg2,psycopg1,psycopg,pygresql'
+            #uriParams['autoCommit'] = '0'
+        if uriParams:
+            uri += "?%s" % "&".join(["=".join(item) for item in uriParams.items()])
+        logger.info('Database: Connecting to DB URI: %s' % uri)
         try:
             self.connection = connectionForURI(uri)
-            self.connection.makeConnection()
+            sqlhub.processConnection = self.connection
         except Exception, inst:
-            msg = 'Database: Could not make a connection: %s' % (inst)
+            msg = "Database: Could not make a connection using DB URI '%s': %s" % (uri, inst)
+            msg += traceback.format_exc()
             logger.critical(msg)
-            raise Exception, "Could not make a database connection (see system log for connection URI: %s)." % self.dictionary[LOG_PATH]
+            raise Exception, "Could not make a database connection (see system log for DB URI and exception: %s)." % self.dictionary[LOG_PATH]
         # Todo: Check connection.dbEncoding == 'UTF8'.
     
     def getConnection(self):
@@ -130,14 +143,14 @@ class DatabaseFacade(object):
             record = None
             if moddebug:
                 logger.info('Database: Finding %s record in database.' % (className))
-            if self.isSelectById(kwds) and len(kwds) == 1:
+            if 'id' in kwds:
                 objectId = kwds['id']
                 try:
                     record = self.get(className, objectId)
                 except SQLObjectNotFound, inst:
                     msg = "Can't find %s record with id %s." \
                         % (className, kwds)
-                    raise KforgeDbError, msg
+                    raise DbObjectNotFound, msg
             else:
                 records = self.findRecords(className, **kwds)
                 if hasattr(records, 'count'):
@@ -146,9 +159,13 @@ class DatabaseFacade(object):
                     except:
                         msg = "Can't find %s record with params %s %s." \
                             % (className, args, kwds)
-                        raise KforgeDbError, msg
+                        raise DbObjectNotFound, msg
                 else:
                     record = records
+            if record == None:
+                msg = "Could't determine %s record with params %s %s." \
+                    % (className, args, kwds)
+                raise Exception, msg
             return record
         
         def isSelectById(self, kwds):
@@ -187,7 +204,7 @@ class DatabaseFacade(object):
                 id = int(id)
             except:
                 msg = "Couldn't cast %s id to int: '%s'" % (className, id)
-                raise Exception, msg
+                raise KforgeRegistryKeyError, msg
             recordClass = self.getRecordClass(className)
             return recordClass.get(id)  # todo: pass transaction
 
@@ -196,26 +213,25 @@ class DatabaseFacade(object):
             if moddebug:
                 logger.info('Database: Finding %s records in database: %s %s' % (className, args, kwds))
             recordClass = self.getRecordClass(className)
-            if self.isSelectById(kwds):
-                records = recordClass.selectByKeywordsWithId(**kwds)
-            elif self.isSelectByTimeInterval(kwds):
-                records = recordClass.selectByKeywordsWithTimeInterval(**kwds)
-            else:    
-                records = recordClass.selectByKeywords(**kwds)
-            return records
+            return recordClass.selectByKwds(**kwds)
 
         def startsWith(self, className, value, attributeName, **kwds):
             recordClass = self.getRecordClass(className)
-            return recordClass.startsWith(value, attributeName, **kwds)
+            kwds['__startsWith__'] = value
+            kwds['__startsWithAttribute__'] = attributeName
+            return recordClass.selectByKwds(**kwds)
 
-        def search(self, className, userQuery, attributeNames, **kwds):
+        def search(self, className, userQuery, attributeNames, spaceSplit=True, **kwds):
             recordClass = self.getRecordClass(className)
-            return recordClass.search(userQuery, attributeNames, **kwds)
+            kwds['__searchQuery__'] = userQuery
+            kwds['__searchQueryAttributes__'] = attributeNames
+            kwds['__searchQuerySpaceSplit__'] = spaceSplit
+            return recordClass.selectByKwds(**kwds)
 
         def countRecords(self, className, *args, **kwds):
             "Counts recorded domain objects."
-            list = self.listRecords(className, *args, **kwds)
-            return list.count()
+            results = self.listRecords(className, *args, **kwds)
+            return results.count()
        
         def listRecords(self, className, *args, **kwds):
             "Returns list of recorded domain objects."
@@ -224,14 +240,14 @@ class DatabaseFacade(object):
             else:
                 records = self.getRecordClass(className).select()
             if moddebug and debug:
-                logger.debug('Listed %s records from database.' % (className))
+                logger.debug('Database: Listed %s records from database.' % (className))
             return records
        
         def createDomainObject(self, className, __loadedList__=None, *args, **kwds):
             "Create new recorded domain object."
             newRecord = self.createRecord(className, *args, **kwds)
             if moddebug and debug:
-                message = "Created new db record. %s" % newRecord
+                message = "Database: Created new record: %s" % newRecord
                 logger.debug(message)
             newObject = newRecord.getDomainObject(loadedList=__loadedList__)
             return newObject
@@ -242,7 +258,7 @@ class DatabaseFacade(object):
             if not self.connection.dictionary[DB_MIGRATION_IN_PROGRESS]:
                 recordClass.assertCreateParams(*args, **kwds)
             if debug:
-                message = "Creating new %s record with %s" % (className, kwds)
+                message = "Database: Creating new %s record with %s" % (className, kwds)
                 logger.debug(message)
             try:
                 newRecord = recordClass(*args, **kwds)
@@ -296,7 +312,8 @@ class DatabaseFacade(object):
                 try:
                     mapperClass.createTable()
                 except Exception, inst:
-                    msg = "Couldn't create table for class '%s': " % className 
+                    msg = "Database: "
+                    msg += "Couldn't create table for class '%s': " % className 
                     msg += "Error: %s. " % inst
                     attributes = str(mapperClass.meta.dom.attributeNames)
                     msg += "Attributes are: %s" % attributes
@@ -304,7 +321,7 @@ class DatabaseFacade(object):
                     raise Exception, msg
                 else: 
                     mapperClass.select().count()
-                    msg = "Added %s table in database." % (
+                    msg = "Database: Added %s table." % (
                         mapperClass.meta.dbName
                     )
                     logger.info(msg)
@@ -339,7 +356,7 @@ class DatabaseFacade(object):
                 delColumn(sqlAttribute, changeSchema=False)
                 addColumn(sqlAttribute, changeSchema=True)
                 list(mapperClass.select())
-                msg = "Added %s field to %s table in database." % (
+                msg = "Database: Added %s field to %s table." % (
                     mapperAttribute.dbName,
                     mapperClass.meta.dbName
                 )
@@ -460,7 +477,7 @@ class MetaBase(object):
         style = self.getSQLStyle()
         safeName = style.pythonClassToDBTable(naiveName)
         if moddebug and debug:
-            self.logger.debug("Made safe db table name: %s" % safeName)
+            self.logger.debug("Database: Made safe table name: %s" % safeName)
         return safeName
     
     def makeSafeFieldName(self, naiveName):
@@ -468,7 +485,7 @@ class MetaBase(object):
         style = self.getSQLStyle()
         safeName = style.pythonAttrToDBColumn(naiveName)
         if moddebug and debug:
-            self.logger.debug("Made safe db field name: %s" % safeName)
+            self.logger.debug("Database: Made safe column name: %s" % safeName)
         return safeName
 
     def makeSafeDbName(self, name):
@@ -514,17 +531,21 @@ class MetaMapper(MetaBase):
                 self.addAttribute(domAttr)
         self.sortOnDbName = None
         if self.dom.sortOn:
-            self.sortOnDbName = self.dom.sortOn.dbName
+            self.sortOnDbName = self.makeSafeFieldName(self.dom.sortOn.dbName)
         self.sortAscending = self.dom.sortAscending
+        self.sortCaseInsensitive = self.dom.sortCaseInsensitive
 
     def getAttribute(self, attrName):
         if not attrName in self.attributeDomNames:
-            msg = "Attribute '%s' not defined on mapper for class '%s'." % (
-                attrName, self.domName
+            msg = "Attribute '%s' not defined on mapper for class '%s': %s" % (
+                attrName, self.domName, self.attributeDomNames.keys()
             )
             raise Exception(msg)
         return self.attributeDomNames[attrName]
 
+    def hasAttribute(self, attrName):
+        return attrName in self.attributeDomNames
+
     def addAttribute(self, metaDomainAttribute):
         metaAttribute = MetaMapperAttribute(metaDomainAttribute)
         self.attributes.append(metaAttribute)
@@ -544,9 +565,11 @@ class MetaMapper(MetaBase):
         if sqoVer == '0.7' and self.dbName:
             mapper.sqlmeta.table = self.dbName
         if self.isCached:
-            mapper.sqlmeta.cacheValues = True
+            mapper.sqlmeta.cacheValues = False
         if self.sortOnDbName:
             defaultOrder = getattr(mapper.q, self.sortOnDbName)
+            if self.sortCaseInsensitive:
+                defaultOrder = func.lower(defaultOrder)
             if not self.sortAscending:
                 defaultOrder = DESC(defaultOrder)
             mapper.sqlmeta.defaultOrder = defaultOrder
@@ -606,7 +629,7 @@ class MetaMapperAttribute(MetaBase):
         self.isValueRef = self.dom.isValueRef
         self.isAssociateList = self.dom.isAssociateList
         self.isDomainObjectRef = self.dom.isDomainObjectRef
-        if sqoVer in ['0.8', '0.9', '0.10', '0.11', '0.12', '0.13', '0.14', '0.15', '1.0'] and self.isDomainObjectRef:
+        if sqoVer in ['0.8', '0.9', '0.10', '0.11', '0.12', '0.13', '0.14', '0.15', '1.0', '1.1', '1.2'] and self.isDomainObjectRef:
             self.typeName = self.makeSafeFieldName(self.dom.typeName)
         else:
             self.typeName = self.dom.typeName
@@ -706,7 +729,7 @@ class MetaMapperAttribute(MetaBase):
         elif issubclass(self.dom.__class__, dm.dom.meta.ImageFile):
             return None
         elif issubclass(self.dom.__class__, dm.dom.meta.DomainObjectRef):
-            if sqoVer in ['0.8', '0.9', '0.10', '0.11', '0.12', '0.13', '0.14', '0.15', '1.0']:
+            if sqoVer in ['0.8', '0.9', '0.10', '0.11', '0.12', '0.13', '0.14', '0.15', '1.0', '1.1', '1.2']:
                 dbName = self.dbName
             elif sqoVer in ['0.7']:
                 dbName = self.dbName + 'ID'
@@ -725,7 +748,7 @@ class Mapper(SQLObject):
     "Maps values between DomainObject and a database."
 
     class sqlmeta:
-        cacheValues = True
+        cacheValues = False
         lazyUpdate = True
 
     domainObject = None
@@ -762,9 +785,9 @@ class Mapper(SQLObject):
         if not kwds:
             return
         if moddebug and debug:
-            msg = "Asserting kwds are unique: %s" % kwds
+            msg = "Database: Asserting kwds are unique: %s" % kwds
             logger.debug(msg)
-        if self.selectByKeywords(**kwds).count():
+        if self.selectByKwds(**kwds).count():
             msg = "'%s' record already exists for: %s" % (self.__name__, kwds)
             logger.info(msg)
             raise KforgeDbError, msg
@@ -836,11 +859,9 @@ class Mapper(SQLObject):
     def loadMapperAttributesFromRecord(self):
         "Maps record values to mapper attributes."
         if moddebug:
-            message = "Database: BOOM! Pulling %s #%s attributes from database." % (self.__class__.__name__, self.id)
+            message = "Database: Pulling %s #%s attributes from database." % (self.__class__.__name__, self.id)
             logger.info(message)
         self.sync()
-        if moddebug:
-            logger.info("Database: DONE")
 
     def loadDomainObjectFromMapperAttributes(self, loadedList):
         if moddebug:
@@ -1049,14 +1070,14 @@ class Mapper(SQLObject):
             currentVersion.save()
         if isChanged:
             if moddebug:
-                message = "Database: BOOM! Putting %s #%s attributes to database." % (self.__class__.__name__, self.id)
+                message = "Database: Saving %s #%s values to database." % (self.__class__.__name__, self.id)
                 logger.info(message)
             try:
                 # Call SQLObject.
                 self.syncUpdate()
             except Exception, inst:
-                message = "Failed to update RDBMS with %s mapper value." % (
-                    self.meta.domName
+                message = "Database: Failed to save database with %s #%s values." % (
+                    self.meta.domName, self.id
                 )
                 logger.error(message)
                 raise
@@ -1065,17 +1086,19 @@ class Mapper(SQLObject):
 
     def coerceKwds(self, kwds):
         "Converting keyword names to database names."
+        if 'id' in kwds:
+            kwds['id'] = self.sqlIdValue(kwds['id'])
         for metaAttr in self.meta.attributes:
             if metaAttr.domName in kwds:
                 if metaAttr.isDomainObjectRef:
                     mapper = kwds[metaAttr.domName]
                     del kwds[metaAttr.domName]
-                    if sqoVer in ['0.10', '0.11', '0.12', '0.13', '0.14', '0.15', '1.0']:
+                    if sqoVer in ['0.10', '0.11', '0.12', '0.13', '0.14', '0.15', '1.0', '1.1', '1.2']:
                         dbName = metaAttr.dbName
                     elif sqoVer in ['0.7', '0.8', '0.9']:
                         dbName = metaAttr.dbName + 'ID'
                     if mapper:
-                        if sqoVer in ['0.10', '0.11', '0.12', '0.13', '0.14', '0.15', '1.0']:
+                        if sqoVer in ['0.10', '0.11', '0.12', '0.13', '0.14', '0.15', '1.0', '1.1', '1.2']:
                             kwds[dbName] = mapper
                         elif sqoVer in ['0.7', '0.8', '0.9']:
                             kwds[dbName] = mapper.id
@@ -1088,229 +1111,99 @@ class Mapper(SQLObject):
 
     coerceKwds = classmethod(coerceKwds)
 
-    def selectByKeywords(self, **kwds):
-        "Selects records from database using domain object attribute names."
+    def selectByKwds(self, **kwds):
         self.coerceKwds(kwds)
-        if moddebug:
-            msg = "Database: BOOM!! Call to selectBy() SQL kwds: %s" % kwds
-            logger.info(msg)
-        try:
-            selection = self.selectBy(**kwds)
-        except Exception, inst:
-            msg = "Couldn't execute selectBy() on '%s' with %s: %s" % (
-                self.meta.domName, kwds, inst
+        # The kwds may contain 'special' parameters to be interpretted.
+        # Try to make SQL conditions from 'special' parameters.
+        # If there are such conditions from 'special' parameters,
+        # then make the normal kwds into SQL conditions.
+        # Otherwise just do selectBy().
+        sqlConditions = []
+        # Pick off 'special' accessedBy kwds.
+        #if 'name' in kwds and kwds['name'] == 'annakarenina':
+        #    raise Exception, str(kwds)
+        if '__accessedBy__' in kwds:
+            accessedBy = kwds.pop('__accessedBy__')
+            if not self.meta.hasAttribute('isHidden'):
+                pass
+            elif not self.meta.hasAttribute('readableBy'):
+                pass
+            else:
+                sqlCondition = self.sqlAccessedBy(accessedBy)
+                sqlConditions.append(sqlCondition)
+        # Pick off 'special' search kwds.
+        if '__searchQuery__' in kwds and '__searchQueryAttributes__' in kwds:
+            sqlCondition = self.sqlSearch(
+                kwds.pop('__searchQuery__'),
+                kwds.pop('__searchQueryAttributes__'),
+                kwds.pop('__searchQuerySpaceSplit__', True),
             )
+            sqlConditions.append(sqlCondition)
+        # Pick off 'special' startsWith kwds.
+        if '__startsWith__' in kwds and '__startsWithAttribute__' in kwds:
+            sqlCondition = self.sqlStartsWith(
+                kwds.pop('__startsWithAttribute__'),
+                kwds.pop('__startsWith__'),
+            )
+            sqlConditions.append(sqlCondition)
+        # Pick off 'special' time interval kwds.
+        for (name, value) in kwds.items():
+            sqlSafeValue = self.sqlRepr(value)
+            sqlCondition = self.sqlTimeInterval(name, sqlSafeValue)
+            if sqlCondition:
+                kwds.pop(name) 
+                sqlConditions.append(sqlCondition)
+        if sqlConditions:
+            # Pick up remaining kwds.
+            for (name, value) in kwds.items():
+                sqlConditions.append(self.sqlEquals(name, value))
+            # Select by condition.
+            sqlCondition = "(%s)" % self.sqlAnd(sqlConditions)
+            logger.debug("Database: Selecting '%s' records by SQL condition: %s" % (self.__name__, sqlCondition))
+            selection = self.selectBySqlCondition(sqlCondition)
+        else:
+            # Default SQLObject selectBy() method.
+            logger.debug("Database: Selecting '%s' records by kwds: %s" % (self.__name__, kwds))
+            selection = self.selectBy(**kwds)
+        logger.debug("Database: Selected %s '%s' records." % (selection.count(), self.__name__))
+        return selection
+
+    selectByKwds = classmethod(selectByKwds)
+
+    def selectBySqlCondition(self, sqlCondition):
+        # Todo: Write test to check special characters are handled OK.
+        logger.debug("Database: Selecting records by SQL condition: %s" % sqlCondition)
+        if type(sqlCondition) == unicode:
+            sqlCondition = sqlCondition.encode('utf8')
+        try:
+            selection = self.select(sqlCondition)
+        except Exception, inst:
+            msg = "Couldn't execute select() with %s: %s" % (sqlCondition, inst)
             raise KforgeDbError, msg
         if moddebug:
             logger.info("Database: DONE")
         return selection
-       
-    selectByKeywords = classmethod(selectByKeywords)
 
-    def startsWith(self, value, attributeName, **kwds):
-        # todo: expand for all database systems
-        dbType = dictionary[DB_TYPE]
-        sqlSafeValue = sqlrepr(value, dbType)[1:-1] # drop quotes
-        dbName = self.getAttributeDbName(attributeName)
-        sqlSafeName = self.makeSqlName(dbName)
-        sqlLike = "UPPER(%s) LIKE UPPER('%s')" % (sqlSafeName, sqlSafeValue+'%')
-        sqlWhere = " ( " + sqlLike + " ) "
-        self.coerceKwds(kwds)
-        for name in kwds:
-            value = kwds[name]
-            if sqoVer in ['0.10', '0.11', '0.12', '0.13', '0.14', '0.15', '1.0']:
-                if self.meta.attributeDbNames[name].isDomainObjectRef:
-                    name += '_id'
-            # todo: expand for all database systems
-            sqlSafeValue = sqlrepr(value, dbType)
-            sqlSafeName = self.makeSqlName(name)
-            if sqlSafeValue.upper() == 'NULL':
-                sqlEquals = "%s IS %s" % (sqlSafeName, sqlSafeValue)
-            else:
-                sqlEquals = "%s = %s" % (sqlSafeName, sqlSafeValue)
-            sqlWhere += " AND ( " + sqlEquals + " ) "
-        sqlWhere = "(" + sqlWhere + ")"
-        return self.selectWhere(sqlWhere)
-        
-    startsWith = classmethod(startsWith)
-
-    def search(self, userQuery, attributeNames, spaceSplit=True, **kwds):
-        dbNames = []
-        for attributeName in attributeNames:
-            dbName = self.getAttributeDbName(attributeName)
-            dbNames.append(dbName)
-        sqlLikeList = []
-        # todo: expand for all database systems
-        dbType = dictionary[DB_TYPE]
-        sqlSafeQuery = sqlrepr(userQuery, dbType)
-        sqlSafeQuery = sqlSafeQuery[1:-1] # Drop quotes (added later).
-        if spaceSplit:
-            sqlSafeTerms = sqlSafeQuery.split(' ')
-        else:
-            sqlSafeTerms = [sqlSafeQuery]
-        for term in sqlSafeTerms:
-            if term:
-                for name in dbNames:
-                    sqlSafeName = self.makeSqlName(name)
-                    sqlLike = "UPPER(%s) LIKE UPPER('%s')" % (
-                        sqlSafeName,'%'+term+'%'
-                    )
-                    sqlLikeList.append(sqlLike)
-            if len(sqlLikeList):
-                sqlLike = "(" + ") OR (".join(sqlLikeList) + ")"
-            else:
-                sqlLike = 'FALSE'
-            
-        self.coerceKwds(kwds)
-        sqlEqualsList = []
-        for name in kwds:
-            value = kwds[name]
-            if sqoVer in ['0.10', '0.11', '0.12', '0.13', '0.14', '0.15', '1.0']:
-                if self.meta.attributeDbNames[name].isDomainObjectRef:
-                    name += '_id'
-            # todo: expand for all database systems
-            sqlSafeValue = sqlrepr(value, dbType)
-            sqlSafeName = self.makeSqlName(name)
-            if sqlSafeValue.upper() == 'NULL':
-                sqlEquals = "%s IS %s" % (sqlSafeName, sqlSafeValue)
-            else:
-                sqlEquals = "%s = %s" % (sqlSafeName, sqlSafeValue)
-            sqlEqualsList.append(sqlEquals)
-        if sqlEqualsList:
-            sqlEquals = "(" + ") AND (".join(sqlEqualsList) + ")"
-            sqlWhere = '((%s) AND (%s))' % (sqlLike, sqlEquals)
-        else:
-            sqlWhere = sqlLike
-        return self.selectWhere(sqlWhere)
-       
-    search = classmethod(search)
-
-    def selectByKeywordsWithId(self, **kwds):
-        self.coerceKwds(kwds)
-        sqlEqualsList = []
-        dbType = dictionary[DB_TYPE]
-        for name in kwds:
-            value = kwds[name]
-            if sqoVer in ['0.10', '0.11', '0.12', '0.13', '0.14', '0.15', '1.0'] and name != 'id':
-                if self.meta.attributeDbNames[name].isDomainObjectRef:
-                    name += '_id'
-            if name == 'id':
-                try:
-                    value = int(value)
-                except:
-                    className = self.getDomainClassName()
-                    msg = "Couldn't cast %s id to int: '%s'" % (className, value)
-                    raise KeyError, msg  # Django templates to carry on.
-            # todo: expand for all database systems
-            sqlSafeValue = sqlrepr(value, dbType)
-            sqlSafeName = self.makeSqlName(name)
-            if sqlSafeValue.upper() == 'NULL':
-                sqlEquals = "%s IS %s" % (sqlSafeName, sqlSafeValue)
-            else:
-                sqlEquals = "%s = %s" % (sqlSafeName, sqlSafeValue)
-            sqlEqualsList.append(sqlEquals)
-        sqlEquals = "(%s)" % ") AND (".join(sqlEqualsList)
-        sqlWhere = '(%s)' % (sqlEquals)
-        return self.selectWhere(sqlWhere)
-
-    selectByKeywordsWithId = classmethod(selectByKeywordsWithId)
-    
-    def selectByKeywordsWithTimeInterval(self, **kwds):
-        self.coerceKwds(kwds)
-        sqlEqualsList = []
-        dbType = dictionary[DB_TYPE]
-        for name in kwds:
-            value = kwds[name]
-            # todo: expand for all database systems
-            sqlSafeValue = sqlrepr(value, dbType)
-            if name == '__startsBefore__':
-                sqlSafeName = self.makeSqlName('starts')
-                sqlEquals = "%s < %s" % (
-                    sqlSafeName, sqlSafeValue
-                )
-                sqlEqualsList.append(sqlEquals)
-            elif name == '__startsAfter__':
-                sqlSafeName = self.makeSqlName('starts')
-                sqlEquals = "%s > %s" % (
-                    sqlSafeName, sqlSafeValue
-                )
-                sqlEqualsList.append(sqlEquals)
-            elif name == '__endsBefore__':
-                sqlSafeName = self.makeSqlName('ends')
-                sqlEquals = "%s < %s" % (
-                    sqlSafeName, sqlSafeValue
-                )
-                sqlEqualsList.append(sqlEquals)
-            elif name == '__endsAfter__':
-                sqlSafeName = self.makeSqlName('ends')
-                sqlEquals = "%s > %s" % (
-                    sqlSafeName, sqlSafeValue
-                )
-                sqlEqualsList.append(sqlEquals)
-            elif name == '__dateCreatedBefore__':
-                sqlSafeName = self.makeSqlName('date_created')
-                sqlEquals = "%s < %s" % (
-                    sqlSafeName, sqlSafeValue
-                )
-                sqlEqualsList.append(sqlEquals)
-            elif name == '__dateCreatedOnOrBefore__':
-                sqlSafeName = self.makeSqlName('date_created')
-                sqlEquals = "%s <= %s" % (
-                    sqlSafeName, sqlSafeValue
-                )
-                sqlEqualsList.append(sqlEquals)
-            elif name == '__dateCreatedAfter__':
-                sqlSafeName = self.makeSqlName('date_created')
-                sqlEquals = "%s > %s" % (
-                    sqlSafeName, sqlSafeValue
-                )
-                sqlEqualsList.append(sqlEquals)
-            elif name == '__lastModifiedBefore__':
-                sqlSafeName = self.makeSqlName('last_modified')
-                sqlEquals = "%s < %s" % (
-                    sqlSafeName, sqlSafeValue
-                )
-                sqlEqualsList.append(sqlEquals)
-            elif name == '__lastModifiedAfter__':
-                sqlSafeName = self.makeSqlName('last_modified')
-                sqlEquals = "%s > %s" % (
-                    sqlSafeName, sqlSafeValue
-                )
-                sqlEqualsList.append(sqlEquals)
-            elif name == '__bookingDateBefore__':
-                sqlSafeName = self.makeSqlName('booking_date')
-                sqlEquals = "%s < %s" % (
-                    sqlSafeName, sqlSafeValue
-                )
-                sqlEqualsList.append(sqlEquals)
-            elif name == '__bookingDateAfter__':
-                sqlSafeName = self.makeSqlName('booking_date')
-                sqlEquals = "%s > %s" % (
-                    sqlSafeName, sqlSafeValue
-                )
-                sqlEqualsList.append(sqlEquals)
-            else:
-                if sqoVer in ['0.10', '0.11', '0.12', '0.13', '0.14', '0.15', '1.0']:
-                    if self.meta.attributeDbNames[name].isDomainObjectRef:
-                        name += '_id'
-                sqlSafeName = self.makeSqlName(name)
-                if sqlSafeValue.upper() == 'NULL':
-                    sqlEquals = "%s IS %s" % (sqlSafeName, sqlSafeValue)
-                else:
-                    sqlEquals = "%s = %s" % (sqlSafeName, sqlSafeValue)
-                sqlEqualsList.append(sqlEquals)
-        sqlEquals = "(%s)" % ") AND (".join(sqlEqualsList)
-        sqlWhere = '(%s)' % (sqlEquals)
-        return self.selectWhere(sqlWhere)
-
-    selectByKeywordsWithTimeInterval = classmethod(
-        selectByKeywordsWithTimeInterval)
+    selectBySqlCondition = classmethod(selectBySqlCondition)
 
     def getAttributeDbName(self, domName):
         return self.meta.getAttribute(domName).dbName
 
     getAttributeDbName = classmethod(getAttributeDbName)
 
+    def sqlRepr(self, value):
+        return sqlrepr(value, dictionary[DB_TYPE])
+
+    sqlRepr = classmethod(sqlRepr)
+
+    def dropQuotes(self, value):
+        if value[0] == "'" and value[-1] == "'":
+            return value[1:-1]
+        else:
+            raise Exception, "Not single quoted: %s" % value
+
+    dropQuotes = classmethod(dropQuotes)
+
     def makeSqlName(self, name):
         if name[-2:] == 'ID':
             name = name[:-2] + '_id'
@@ -1318,21 +1211,131 @@ class Mapper(SQLObject):
 
     makeSqlName = classmethod(makeSqlName)
     
-    def selectWhere(self, sqlWhere):
-        # Todo: Write test to check special character are handled OK.
-        if type(sqlWhere) == unicode:
-            sqlWhere = sqlWhere.encode('utf8')
-        if moddebug:
-            msg = "Database: BOOM!! Call to select() SQL WHERE: %s" % sqlWhere
-            logger.info(msg)
-        try:
-            selection = self.select(sqlWhere)
-        except Exception, inst:
-            msg = "Couldn't execute select() with %s: %s" % (sqlWhere, inst)
-            raise KforgeDbError, msg
-        if moddebug:
-            logger.info("Database: DONE")
-        return selection
+    def sqlEquals(self, name, value):
+        if self.meta.attributeDbNames[name].isDomainObjectRef:
+            name += '_id'
+        sqlSafeName = self.makeSqlName(name)
+        sqlSafeValue = self.sqlRepr(value)
+        if sqlSafeValue.upper() == 'NULL':
+            sqlCondition = "%s IS %s" % (sqlSafeName, sqlSafeValue)
+        else:
+            sqlCondition = "%s = %s" % (sqlSafeName, sqlSafeValue)
+        return sqlCondition
 
-    selectWhere = classmethod(selectWhere)
+    sqlEquals = classmethod(sqlEquals)
+
+    def sqlAnd(self, sqlConditions):
+        return "(%s)" % ") AND (".join(sqlConditions)
+
+    sqlAnd = classmethod(sqlAnd)
+
+    def sqlOr(self, sqlConditions):
+        return "(%s)" % ") OR (".join(sqlConditions)
+
+    sqlOr = classmethod(sqlOr)
+
+    def sqlTimeInterval(self, name, sqlSafeValue):
+        sqlCondition = None
+        if name == '__startsBefore__':
+            sqlSafeName = self.makeSqlName('starts')
+            sqlCondition = "%s < %s" % (sqlSafeName, sqlSafeValue)
+        elif name == '__startsAfter__':
+            sqlSafeName = self.makeSqlName('starts')
+            sqlCondition = "%s > %s" % (sqlSafeName, sqlSafeValue)
+        elif name == '__endsBefore__':
+            sqlSafeName = self.makeSqlName('ends')
+            sqlCondition = "%s < %s" % (sqlSafeName, sqlSafeValue)
+        elif name == '__endsAfter__':
+            sqlSafeName = self.makeSqlName('ends')
+            sqlCondition = "%s > %s" % (sqlSafeName, sqlSafeValue)
+        elif name == '__dateCreatedBefore__':
+            sqlSafeName = self.makeSqlName('date_created')
+            sqlCondition = "%s < %s" % (sqlSafeName, sqlSafeValue)
+        elif name == '__dateCreatedOnOrBefore__':
+            sqlSafeName = self.makeSqlName('date_created')
+            sqlCondition = "%s <= %s" % (sqlSafeName, sqlSafeValue)
+        elif name == '__dateCreatedAfter__':
+            sqlSafeName = self.makeSqlName('date_created')
+            sqlCondition = "%s > %s" % (sqlSafeName, sqlSafeValue)
+        elif name == '__lastModifiedBefore__':
+            sqlSafeName = self.makeSqlName('last_modified')
+            sqlCondition = "%s < %s" % (sqlSafeName, sqlSafeValue)
+        elif name == '__lastModifiedAfter__':
+            sqlSafeName = self.makeSqlName('last_modified')
+            sqlCondition = "%s > %s" % (sqlSafeName, sqlSafeValue)
+        elif name == '__bookingDateBefore__':
+            sqlSafeName = self.makeSqlName('booking_date')
+            sqlCondition = "%s < %s" % (sqlSafeName, sqlSafeValue)
+        elif name == '__bookingDateAfter__':
+            sqlSafeName = self.makeSqlName('booking_date')
+            sqlCondition = "%s > %s" % (sqlSafeName, sqlSafeValue)
+        return sqlCondition
+
+    sqlTimeInterval = classmethod(sqlTimeInterval)
     
+    def sqlAccessedBy(self, accessedBy):
+        sqlNotHidden = self.sqlEquals(self.getAttributeDbName('isHidden'), False)
+        if accessedBy == None:
+            accessedBy = ''
+        sqlReadableBy = self.sqlSearchTerm('readableBy', ':%s:' % accessedBy)
+        return self.sqlOr([sqlNotHidden, sqlReadableBy])
+        
+    sqlAccessedBy = classmethod(sqlAccessedBy)
+
+    def sqlSearch(self, userQuery, attributeNames, spaceSplit):
+        queryTerms = userQuery.split(' ') if spaceSplit else [userQuery]
+        sqlConditions = []
+        for queryTerm in queryTerms:
+            if not queryTerm:
+                continue
+            if ':' in queryTerm:
+                # Attribute specific term.
+                name, value = queryTerm.split(':', 1)
+                if name and value and name in attributeNames:
+                    sqlCondition = self.sqlSearchTerm(name, value)
+                    sqlConditions.append(sqlCondition)
+            else:
+                # Non-attribute specific term.
+                subConditions = []
+                for name in attributeNames:
+                    value = queryTerm
+                    subCondition = self.sqlSearchTerm(name, value)
+                    subConditions.append(subCondition)
+                if subConditions:
+                    sqlCondition = self.sqlOr(subConditions)
+                    sqlConditions.append(sqlCondition)
+        if not len(sqlConditions):
+            sqlConditions.append('id=NULL')
+        #raise Exception, self.sqlAnd(sqlConditions)
+        return self.sqlAnd(sqlConditions)
+        
+    sqlSearch = classmethod(sqlSearch)
+
+    def sqlSearchTerm(self, attributeName, value):
+        dbName = self.getAttributeDbName(attributeName)
+        sqlSafeName = self.makeSqlName(dbName)
+        sqlSafeValue = self.dropQuotes(self.sqlRepr(value))
+        return "UPPER(%s) LIKE UPPER('%%%s%%')" % (sqlSafeName, sqlSafeValue)
+
+    sqlSearchTerm = classmethod(sqlSearchTerm)
+
+    def sqlStartsWith(self, attributeName, value):
+        dbName = self.getAttributeDbName(attributeName)
+        sqlSafeName = self.makeSqlName(dbName)
+        sqlSafeValue = self.dropQuotes(self.sqlRepr(value))
+        return "UPPER(%s) LIKE UPPER('%s%%')" % (sqlSafeName, sqlSafeValue)
+
+    sqlStartsWith = classmethod(sqlStartsWith)
+
+    def sqlIdValue(self, value):
+        try:
+            value = int(value)
+        except:
+            className = self.getDomainClassName()
+            msg = "Couldn't cast %s id to int: '%s'" % (className, value)
+            raise KforgeRegistryKeyError, msg  # Django templates to carry on.
+        return value
+   
+    sqlIdValue = classmethod(sqlIdValue)
+
+
