@@ -6,8 +6,9 @@ from dm.dom.meta import *
 from dm.exceptions import *
 import dm.times
 from dm.util.datastructure import MultiValueDict
-from dm.dictionarywords import DB_MIGRATION_IN_PROGRESS
+from dm.dictionarywords import *
 import traceback
+import weakref
 
 debug = RequiredFeature('Debug')
 moddebug = False
@@ -17,15 +18,23 @@ class DomainBase(object):
     'Layer Supertype (475)'  [Fowler, 2003]
     """
 
-    log              = RequiredFeature('Logger')
+    log              = RequiredFeature('Logger')  # Todo: Change all references to use 'logger', and then remove 'log'.
+    logger           = RequiredFeature('Logger')
     dictionary       = RequiredFeature('SystemDictionary')
     registry         = RequiredFeature('DomainRegistry')
     pluginController = RequiredFeature('PluginController')
 
     isObjectIdSignificant = True  # For object data migration.
 
-    def resolvePathPart(self, pathPart):
-        return getattr(self, pathPart)
+    def resolvePathPart(self, pathPart, **kwds):
+        if hasattr(self, pathPart):
+            attrValue = getattr(self, pathPart)
+            if isinstance(attrValue, DomainBase):
+                return attrValue
+        raise KforgeAttributeError
+
+    def getUri(self, absoluteUriPrefix):
+        raise Exception, "Method not implemented on %s" % self.__class__
 
 
 class AbstractRegister(DomainBase): 
@@ -37,8 +46,10 @@ class AbstractRegister(DomainBase):
     database = RequiredFeature('DatabaseFacade')
 
     def __init__(self, owner=None, ownerName=None, owner2=None,
-            ownerName2=None, filter={}, isCached=False, **kwds):
+            ownerName2=None, filter={}, isCached=False,
+            ownerAsOwnerAttrName='', **kwds):
         self.owner = owner
+        self.ownerAsOwnerAttrName = ownerAsOwnerAttrName
         self.ownerName = ownerName
         self.owner2 = owner2
         self.ownerName2 = ownerName2
@@ -65,7 +76,7 @@ class AbstractRegister(DomainBase):
 
     def has_key(self, key):
         if self.isCached:
-            if key in self.cache:
+            if key in self.cache and self.cache[key] != None:
                 return True
         try:
             item = self.find(key)
@@ -74,22 +85,36 @@ class AbstractRegister(DomainBase):
         else:
             return True
        
-    def find(self, key):
+    def find(self, key, **kwds):
         "Returns keyed object."
         if self.isCached:
+            msg = "Model cache: %s class is cached." % self.typeName
+            self.log.debug(msg)
             if key in self.cache:
-                if moddebug:
-                    msg = "Model: Cache hit for key: '%s'" % key
-                    self.log.info(msg)
-                return self.readCache(key)
+                msg = "Model cache: Cache hit for key: '%s'" % key
+                self.log.debug(msg)
+                item = self.readCache(key)
+                if item == None:
+                    msg = "Model cache: %s object previously not found for key '%s'." % (
+                        self.typeName, key
+                    )
+                    self.log.debug(msg)
+                    raise KforgeRegistryKeyError(msg)
+                else:
+                    return item
             else:
-                if moddebug:
-                    msg = "Model: Cache miss for key: '%s'" % key
-                    self.log.info(msg)
-        if moddebug:
-            msg = "Model: Retrieving from %s record for key: '%s'" % (self, key)
-            self.log.info(msg)
-        return self.retrieveItem(key)
+                msg = "Model cache: Cache miss for key: '%s'" % key
+                self.log.debug(msg)
+        else:
+            msg = "Model cache: %s class is not cached." % self.typeName
+            self.log.debug(msg)
+        msg = "Model: Retrieving object for key '%s' from register %s." % (key, self)
+        self.log.debug(msg)
+        domainObject = self.retrieveItem(key, **kwds)
+        if domainObject:
+            msg = "Model: Retrieved object for key '%s': %s" % (key, domainObject)
+            self.log.debug(msg)
+        return domainObject
 
     def retrieveItem(self, *args, **kwds):
         "Abstract method to retrieve register entity."
@@ -134,15 +159,42 @@ class DomainObjectRegister(AbstractRegister):
         # Meta object for supported model attribute.
         self.metaAttr = metaAttr
         # Attributes used in basic search.
-        if 'searchAttributeNames' in kwds:
-            self.searchAttributeNames = kwds['searchAttributeNames']
-        else:
-            self.searchAttributeNames = ''
-        if 'startsWithAttributeName' in kwds:
-            self.startsWithAttributeName = kwds['startsWithAttributeName']
-        else:
-            self.startsWithAttributeName = ''
+        self.searchAttributeNames = kwds.get('searchAttributeNames', [])
+        self.startsWithAttributeName = kwds.get('startsWithAttributeName', '')
         
+    def getUris(self, absoluteUriPrefix=None, **kwds):
+        # Obtain URIs for the domain objects in this register.
+        if self.getDomainClass().isImplicitAssociation:
+            if self.getKeyMeta() and self.getKeyMeta().isDomainObjectRef:
+                objects = self.keys(**kwds)
+            else:
+                raise Exception, "Key is not a domain object, although domain class is an implicit association."
+        else:
+            objects = self.findDomainObjects(**kwds)
+        return [str(i.getUri(absoluteUriPrefix)) for i in objects]
+        #keys = self.keys()
+        #if self.getKeyMeta() and self.getKeyMeta().isDomainObjectRef:
+        #    uris = [k.getRegisterKeyValue() for k in keys]
+        #else:
+        #    uris = keys
+        #if absoluteUriPrefix:
+        #    prefix = self.getUri(absoluteUriPrefix)
+        #    uris = [prefix + '/' + unicode(i) for i in uris]
+        #return uris
+
+    def getUri(self, absoluteUriPrefix=None):
+        if self.metaAttr:
+            registerUri = self.metaAttr.name
+        else:
+            registerUri = self.getDomainClass().getRegistryAttrName()
+        if self.owner:
+            ownerUri = self.owner.getUri(absoluteUriPrefix)
+        elif absoluteUriPrefix:
+            ownerUri = absoluteUriPrefix
+        else:
+            ownerUri = ''
+        return ownerUri + '/' + registerUri
+ 
     def __repr__(self):
         className = self.__class__.__name__
         typeName = self.typeName
@@ -151,41 +203,54 @@ class DomainObjectRegister(AbstractRegister):
             className, typeName, keyName
         )
 
-    def resolvePathPart(self, pathPart):
+    def resolvePathPart(self, pathPart, **kwds):
         # Make register key from path part.
         keyMeta = self.getKeyMeta()
         if keyMeta == None:
             keyValue = pathPart
         elif keyMeta.isDomainObjectRef:
-            keyValue = keyMeta.getObjectByKey(pathPart)
+            keyValue = keyMeta.getObjectByKey(pathPart, **kwds)
         else:
             keyValue = pathPart
-        return self[keyValue]
+        register = self
+        # Todo: Fix data migration dumping, something up with deleted items.
+        if self.dictionary[DB_MIGRATION_IN_PROGRESS]:
+            if keyValue == 'deleted' and hasattr(self, 'deleted'):
+                return register
+            elif keyValue == 'pending' and hasattr(self, 'pending'):
+                return register
+            elif hasattr(self, 'all'):
+                register = register.all
+        return register.find(keyValue, **kwds)
 
-    def retrieveItem(self, key):
+    def retrieveItem(self, key, **kwds):
         "Attempts to retrieve existing domain object from database."
         item = None
         try:
-            item = self.findDomainObject(key)
-            if item and self.isCached:
-                self.cacheItem(item)
+            item = self.findDomainObject(key, **kwds)
             if not item:
-                message = "findDomainObject() returned: %s" % unicode(item)
-                raise StandardError(message)
-        except StandardError, inst:
-            #trace = traceback.format_exc()
-            #self.log.error("Model: Error calling findDomainObject() method: %s" % trace)
-            msg = "No %s called '%s' found: %s" \
-                % (self.typeName, key, inst)
+                message = "findDomainObject() returned nothing"
+                raise Exception(message)
+        except DbObjectNotFound, inst:
+            msg = "Model: %s object not found for key '%s': %s" % (
+                self.typeName, key, inst
+            )
+            self.log.debug(msg)
+            if self.isCached:
+                self.cacheItem(None, key=key)
             raise KforgeRegistryKeyError(msg)
+        if self.isCached:
+            self.cacheItem(item)
         return item
 
     def create(self, *args, **kwds):
         "Returns new registered domain object."
         item = self.createDomainObject(*args, **kwds)
+        if self.isCached:
+            self.cacheItem(item)
         if item:
-            if self.isCached:
-                self.cacheItem(item)
+            #if self.isCached:
+            #    self.cacheItem(item)
             item.raiseCreate()
         return item
 
@@ -198,15 +263,17 @@ class DomainObjectRegister(AbstractRegister):
         self.coerceKwds(kwds)
         self.switchToRecords(kwds)
         results = self.database.listRecords(self.typeName, **kwds)
+        results = list(results)
         iterator = RegisterIterator(results=results, register=self)
         objectList = [i for i in iterator]
-        self.sortDomainObjects(objectList)
+        if not self.getDomainClass().sortOnName:
+            self.sortDomainObjects(objectList)
         return iter(objectList)
  
     def __delitem__(self, key):
         "Supports: del registry[key]"
         self.pop(key)
- 
+
     def pop(self, key):
         item = self[key]
         item.delete()
@@ -228,12 +295,11 @@ class DomainObjectRegister(AbstractRegister):
         "Returns list of records of register."
         self.coerceKwds(kwds)
         self.switchToRecords(kwds)
-        values = []
         results = self.database.listRecords(self.typeName, **kwds)
+        results = list(results)
         return [r.getDomainObject() for r in results]
 
-    def startsWith(self, value, attributeName=''):
-        kwds = {}
+    def startsWith(self, value, attributeName='', **kwds):
         self.coerceKwds(kwds)
         self.switchToRecords(kwds)
         if not attributeName:
@@ -241,24 +307,24 @@ class DomainObjectRegister(AbstractRegister):
         results = self.database.startsWith(
             self.typeName, value, attributeName, **kwds
         )
-        #return RegisterIterator(results=results, register=self)
         iterator = RegisterIterator(results=results, register=self)
         objectList = [i for i in iterator]
-        self.sortDomainObjects(objectList)
+        if not self.getDomainClass().sortOnName:
+            self.sortDomainObjects(objectList)
         return objectList
     
-    def search(self, userQuery, attributeNames=None, spaceSplit=True):
-        kwds = {'spaceSplit':spaceSplit}
+    def search(self, userQuery, attributeNames=None, spaceSplit=True, **kwds):
+        kwds['spaceSplit'] = spaceSplit
         self.coerceKwds(kwds)
         self.switchToRecords(kwds)
         if not attributeNames:
             attributeNames = self.getSearchAttributeNames()
-        results = self.database.search(
-            self.typeName, userQuery, attributeNames, **kwds
-        )
+        results = self.database.search(self.typeName, userQuery,
+                                       attributeNames, **kwds)
         iterator = RegisterIterator(results=results, register=self)
         objectList = [i for i in iterator]
-        self.sortDomainObjects(objectList)
+        if not self.getDomainClass().sortOnName:
+            self.sortDomainObjects(objectList)
         return objectList
     
     def createDomainObject(self, *args, **kwds):
@@ -284,9 +350,8 @@ class DomainObjectRegister(AbstractRegister):
         else:
             return domainObject
 
-    def findDomainObject(self, key):
+    def findDomainObject(self, key, **kwds):
         "Finds existing object."
-        kwds = dict()
         keyName = self.getKeyName()
         if keyName:
             kwds[keyName] = key
@@ -329,19 +394,23 @@ class DomainObjectRegister(AbstractRegister):
         for record in self.findRecords(**kwds):
             domainObject = record.getDomainObject(__loadedList__)
             objectList.append(domainObject)
-        self.sortDomainObjects(objectList)
+        if not self.getDomainClass().sortOnName:
+            self.sortDomainObjects(objectList)
         return objectList
     
     def findRecord(self, **kwds):
         "Finds existing object record."
         self.coerceKwds(kwds)
         self.switchToRecords(kwds)
+        msg = "Model: Finding %s record: %s" % (self.typeName, kwds)
+        self.log.debug(msg)
         return self.database.findRecord(self.typeName, **kwds)
 
     def findRecords(self, **kwds):
         "Finds existing object records."
         self.coerceKwds(kwds)
         self.switchToRecords(kwds)
+        msg = "Model: Finding %s records: %s" % (self.typeName, kwds)
         return self.database.findRecords(self.typeName, **kwds)
 
     def coerceArgs(self, args, kwds):
@@ -371,8 +440,7 @@ class DomainObjectRegister(AbstractRegister):
         
     def initialiseWithDefaults(self, kwds):
         for metaAttr in self.getDomainClassMetaAttributes():
-            attrName = metaAttr.name
-            if attrName in kwds:
+            if metaAttr.name in kwds:
                 continue
             if metaAttr.isDomainObjectRef:
                 if not (hasattr(metaAttr, 'default') and metaAttr.default):
@@ -381,16 +449,25 @@ class DomainObjectRegister(AbstractRegister):
                 attrClass = self.registry.getDomainClass(attrClassName)
                 attrDomainRegister = attrClass.createRegister()
                 attrValue = attrDomainRegister[metaAttr.default]
-                kwds[attrName] = attrValue
+                kwds[metaAttr.name] = attrValue
 
     def coerceKwds(self, kwds):
         if self.owner and self.ownerName:
-            kwds[self.ownerName] = self.owner
+            if self.ownerAsOwnerAttrName:
+                owner = getattr(self.owner, self.ownerAsOwnerAttrName)
+            else:
+                owner = self.owner
+            kwds[self.ownerName] = owner
         if self.owner2 and self.ownerName2:
             kwds[self.ownerName2] = self.owner2
         for (k, v) in self.filter.items():
             if k not in kwds:
                 kwds[k] = v
+        if '__viewer__' in kwds:
+            viewer = kwds.pop('__viewer__')
+            viewerName = viewer.name if viewer else ''
+            if viewerName:
+                kwds['__viewerName__'] = viewerName
 
     def switchToRecords(self, kwds):
         metaAttributeNames = self.getDomainClassMetaAttributeNames()
@@ -408,14 +485,18 @@ class DomainObjectRegister(AbstractRegister):
                             )
                             raise KforgeRegistryKeyError, msg
                 
-    def initialiseObject(self, object):
-        object.initialise(self)
+    def initialiseObject(self, domainObject):
+        domainObject.initialise(self)
 
-    def cacheItem(self, item):
+    def cacheItem(self, item, key=None):
         "Adds item to in-memory cache."
-        key = self.getRegisterKey(item)
+        msg = "Model cache: Cacheing object: %s" % (item)
+        self.log.debug(msg)
+        if key == None:
+            key = self.getRegisterKey(item)
         self.cache[key] = item
-        item.cacheRegister(self)
+        if item != None:
+            item.cacheRegister(self)
     
     def readCache(self, item):
         "Reads item from in-memory cache."
@@ -423,6 +504,8 @@ class DomainObjectRegister(AbstractRegister):
     
     def decacheItem(self, item):
         "Removes item from in-memory cache."
+        msg = "Model cache: Decacheing object: %s" % (item)
+        self.log.debug(msg)
         item.decacheRegister(self)
         key = self.getRegisterKey(item)
         self.decacheKey(key)
@@ -458,10 +541,21 @@ class DomainObjectRegister(AbstractRegister):
 
     def getStartsWithAttributeName(self):
         "Returns name of domain object attribute used in alphabetical index."
-        attrName = self.startsWithAttributeName or self.getKeyName()
+        attrName = self.startsWithAttributeName
+        if not attrName:
+            sortOnName = self.getDomainClass().sortOnName
+            if sortOnName != 'id':
+                attrName = sortOnName
+        if not attrName:
+             keyName = self.getKeyName()
+             if keyName != 'id':
+                 attrName = keyName
         if attrName == 'id':
-            msg = "Can't match start of 'id' attribute. Try setting the " 
-            msg += "'searchAttributeNames' or 'startsWithAttributeName' "
+            msg = "Can't use 'id' for 'starts with' attribute name "
+            msg += "on class %s." % self.getDomainClassMeta().name
+            raise Exception(msg)
+        if not attrName:
+            msg = "Can't determine 'starts with' attribute name "
             msg += "on class %s." % self.getDomainClassMeta().name
             raise Exception(msg)
         return attrName
@@ -558,6 +652,7 @@ class DomainObject(DomainBase):
     Domain object supertype. Follows 'Domain Model (116)'. 
     """
 
+    collectiveNoun = None
     registerKeyName = 'id'
     registerClass = Register
     principalRegister = None
@@ -567,14 +662,18 @@ class DomainObject(DomainBase):
     isConstant = False
     isUnique = True
     isTemporal = False
+    isImplicitAssociation = False
     searchAttributeNames = []
     startsWithAttributeName = ''
     isRegistered = False
     dbName = ''
     registryAttrName = None
-    sortOnName = None 
+    ownerAttrNames = []
+    sortCaseInsensitive = False
+    sortOnName = 'id'
     sortAscending = True
     nextAscends = True
+    hasModelExtn = False
 
     def __init__(self, **kwds):
         "Initialise base attributes."
@@ -587,7 +686,7 @@ class DomainObject(DomainBase):
         if self.meta:
             for metaAttr in self.meta.attributes:
                 metaAttr.setInitialValue(self)
-                
+
     ## experimental
     #def __setattr__(self, attrName, attrVal):
     #    if self.meta and self.meta.attributeNames.has_key(attrName) and hasattr(self, 'objectAttrValues'):
@@ -609,6 +708,47 @@ class DomainObject(DomainBase):
         reprAttrs = self.reprAttrs()
         return "<%s%s>" % (className, reprAttrs)
 
+    def getUri(self, absoluteUriPrefix=None):
+        parts = []
+        if self.ownerAttrNames:
+            # Follow model attributes to obtain owner.
+            ownerObject = getattr(self, self.ownerAttrNames[0])
+            if len(self.ownerAttrNames) == 1:
+                ownerUri = ownerObject.getUri(absoluteUriPrefix)
+                parts.append(ownerUri)
+            elif len(self.ownerAttrNames) == 2:
+                ownerRegister = getattr(ownerObject, self.ownerAttrNames[1])
+                if ownerRegister.isStateful and ownerRegister.getRequiredState() != self.state:
+                    ownerRegister = getattr(ownerRegister, self.state.name)
+                ownerUri = ownerRegister.getUri(absoluteUriPrefix)
+                keyValue = ownerRegister.getRegisterKey(self)
+                if isinstance(keyValue, DomainObject):
+                    keyValue = keyValue.getRegisterKeyValue()
+                parts.append(ownerUri)
+                parts.append(keyValue)
+            else:
+                raise Exception, "Can't handle more than 2 'ownerAttrNames': %s" % self.ownerAttrNames
+        elif self.principalRegister:
+            # Assume containing register is the principal register.
+            ownerUri = self.principalRegister.getUri(absoluteUriPrefix)
+            keyValue = self.getRegisterKeyValue()
+            parts.append(ownerUri)
+            parts.append(keyValue)
+        else:
+            ownerRegister = self.createRegister()
+            ownerUri = ownerRegister.getUri(absoluteUriPrefix)
+            keyValue = self.getRegisterKeyValue()
+            parts.append(ownerUri)
+            parts.append(keyValue)
+        return '/'.join([str(i) for i in parts])
+
+    def resolvePathPart(self, pathPart, **kwds):
+        # Raise attribute error for implicit associations.
+        metaAttr = self.meta.attributeNames.get(pathPart, None)
+        if metaAttr and metaAttr.isAssociateList and metaAttr.getDomainClass().isImplicitAssociation:
+            raise KforgeAttributeError
+        return super(DomainObject, self).resolvePathPart(pathPart, **kwds)
+
     def reprAttrs(self):
         reprAttrs = " id='%d'" % self.id
         if self.meta:
@@ -628,28 +768,35 @@ class DomainObject(DomainBase):
             sortOnValue = sortOnValue.lower()
         return sortOnValue
 
-    def asNamedValues(self, excludeName=''):  # todo: excludeNames
+    def asNamedValues(self, attrNames=None, excludeName=''):  # todo: excludeNames
         namedValues = [] 
-        if self.meta:
-            for attr in self.meta.attributes:
-                if not excludeName == attr.name:
-                    namedValue = {}
-                    namedValue['name'] = attr.name
-                    namedValue['title'] = attr.calcTitle()
-                    namedValue['value'] = attr.createValueRepr(self)
-                    namedValue['label'] = attr.createLabelRepr(self)
-                    namedValue['domainKeyValue'] = self.getRegisterKeyValue()
-                    namedValue['isDomainObjectRef'] = attr.isDomainObjectRef
-                    if attr.isDomainObjectRef:
-                        namedValue['isSimpleOption'] = attr.isSimpleOption
+        if attrNames == None:
+            attrNames = self.meta.attributeNames.keys()
+        for attrName in attrNames:
+            attr = self.meta.attributeNames[attrName]
+            if attr.name != excludeName:
+                namedValue = {}
+                namedValue['name'] = attr.name
+                namedValue['title'] = attr.calcTitle()
+                namedValue['value'] = attr.createValueRepr(self)
+                namedValue['label'] = attr.createLabelRepr(self)
+                #namedValue['domainKeyValue'] = self.getRegisterKeyValue()
+                namedValue['isDomainObjectRef'] = attr.isDomainObjectRef
+                if attr.isDomainObjectRef:
+                    domainObject = getattr(self, attr.name)
+                    if domainObject:
+                        namedValue['registerKeyValue'] = domainObject.getRegisterKeyValue()
                     else:
-                        namedValue['isSimpleOption'] = False
-                    namedValue['registryAttrName'] = attr.getRegistryAttrName(self)
-                    namedValue['typeName'] = attr.typeName
-                    namedValue['isAssociateList'] = attr.isAssociateList
-                    if attr.isAssociateList:
-                        namedValue['associatedNamedValues'] = attr.createAssociatedNamedValues(self)
-                    namedValues.append(namedValue)
+                        namedValue['registerKeyValue'] = ''
+                    namedValue['isSimpleOption'] = attr.isSimpleOption
+                else:
+                    namedValue['isSimpleOption'] = False
+                namedValue['registryAttrName'] = attr.getRegistryAttrName(self)
+                namedValue['typeName'] = attr.typeName
+                namedValue['isAssociateList'] = attr.isAssociateList
+                if attr.isAssociateList:
+                    namedValue['associatedNamedValues'] = attr.createAssociatedNamedValues(self)
+                namedValues.append(namedValue)
         return namedValues
 
     def asSortableValues(self):
@@ -659,17 +806,19 @@ class DomainObject(DomainBase):
                 values[attr.name] = attr.createSortableRepr(self)
         return values
 
-    def asDictValues(self, attrNames=None):
+    def asDictValues(self, attrNames=None, absoluteUriPrefix=None):
+        dictValues = {}
         if attrNames == None:
             attrNames = self.meta.attributeNames.keys()
-        dictValues = {}
         for attrName in attrNames:
             attr = self.meta.attributeNames[attrName]
-            dictValues[attrName] = attr.createValueRepr(self)
+            dictValues[attrName] = attr.createValueRepr(self, absoluteUriPrefix=absoluteUriPrefix)
+        if self.__class__.registerKeyName == 'id':
+            dictValues['id'] = self.id
         return dictValues
 
     def asDictLabels(self):
-        labels = {} 
+        labels = {}
         if self.meta:
             for attr in self.meta.attributes:
                 labels[attr.name] = attr.createLabelRepr(self)
@@ -723,19 +872,14 @@ class DomainObject(DomainBase):
 
     def saveSilently(self):
         "Update record without raising update event."
-        if not self.dictionary[DB_MIGRATION_IN_PROGRESS]:
-            self.updateLastModifiedTime()
         self.record.saveDomainObject()
 
-    def updateLastModifiedTime(self):
-        if 'lastModified' in self.meta.attributeNames:
-            self.lastModified = dm.times.getUniversalNow()
-        
     def delete(self):
         self.deleteAggregates()
         # Todo: Set 'HasA' on normal collection to 'None'?
         self.raiseDelete()
         self.destroySelf()
+        self.raiseAfterDelete()
 
     def deleteAggregates(self):
         for metaAttr in self.meta.attributes:
@@ -756,6 +900,8 @@ class DomainObject(DomainBase):
             self.record.domainObject = None
             self.record.destroySelf()
             self.record = None
+        #else:
+        #    raise Exception, "Domain object has no record to destroy."
 
     def deleteTemporalObjects(self):
         if self.meta.isTemporal:
@@ -780,13 +926,26 @@ class DomainObject(DomainBase):
         for aggregateObject in aggregateRegister:
             aggregateObject.purge()
 
+    def dropHasAs(self):
+        for metaAttr in self.meta.attributes:
+            if metaAttr.isDomainObjectRef:
+                setattr(self, metaAttr.name, None)
+
     def decacheItem(self):
         for register in self.registerCache.keys():
             register.decacheItem(self)
 
     def notifyPlugins(self, *args):
-        if self.pluginController:
-            self.pluginController.notify(*args)
+        errorAttrName = '_hasNotificationError'
+        if (hasattr(self, errorAttrName) and getattr(self, errorAttrName)):
+            self.log.error("Model: Notification of event suspended due to previous notification error.")
+        else:
+            try:
+                if self.pluginController:
+                    self.pluginController.notify(*args)
+            except:
+                setattr(self, errorAttrName, True)
+                raise
 
     def raiseCreate(self):
         "Raises onCreate event."
@@ -821,7 +980,7 @@ class DomainObject(DomainBase):
                 self.__class__.__name__,
                 self
             )
-            self.log.debug(message)
+            self.log.info(message)
         try:
             self.onUpdate()
         except Exception, inst:
@@ -831,7 +990,7 @@ class DomainObject(DomainBase):
 
     def raiseDelete(self):
         "Raises onDelete event."
-        message = "Model: Deleted %s: '%s'" % (
+        message = "Model: Deleting %s: '%s'" % (
             self.__class__.__name__,
             self.getRegisterKeyValue()
         )
@@ -841,7 +1000,21 @@ class DomainObject(DomainBase):
         except Exception, inst:
             trace = traceback.format_exc()
             self.log.error("Model: Error raising delete event: %s" % trace)
-        # Don't re-raise event, because it trips up undoing create.
+            raise
+
+    def raiseAfterDelete(self):
+        "Raises onAfterDelete event."
+        message = "Model: Deleted %s: '%s'" % (
+            self.__class__.__name__,
+            self.getRegisterKeyValue()
+        )
+        self.log.info(message)
+        try:
+            self.onAfterDelete()
+        except Exception, inst:
+            trace = traceback.format_exc()
+            self.log.error("Model: Error raising after-delete event: %s" % trace)
+            raise
 
     def raiseApprove(self):
         "Raises onApprove event."
@@ -887,27 +1060,31 @@ class DomainObject(DomainBase):
 
 
     def onCreate(self):
-        "Abstract handler for Create object event."
+        "Default behaviour for Create object event."
         self.notifyPlugins(self.__class__.__name__ + 'Create', self)
 
     def onUpdate(self):
-        "Abstract handler for Update event."
+        "Default behaviour for Update event."
         self.notifyPlugins(self.__class__.__name__ + 'Update', self)
 
     def onDelete(self):
-        "Abstract handler for Delete object event."
+        "Default behaviour for Delete object event."
         self.notifyPlugins(self.__class__.__name__ + 'Delete', self)
     
+    def onAfterDelete(self):
+        "Default behaviour for AfterDelete object event."
+        self.notifyPlugins(self.__class__.__name__ + 'AfterDelete', self)
+    
     def onApprove(self):
-        "Abstract handler for Approve object event."
+        "Default behaviour for Approve object event."
         self.notifyPlugins(self.__class__.__name__ + 'Approve', self)
     
     def onUndelete(self):
-        "Abstract handler for Undelete object event."
+        "Default behaviour for Undelete object event."
         self.notifyPlugins(self.__class__.__name__ + 'Undelete', self)
     
     def onPurge(self):
-        "Abstract handler for Purge object event."
+        "Default behaviour for Purge object event."
         self.notifyPlugins(self.__class__.__name__ + 'Purge', self)
 
     def cacheRegister(self, register):
@@ -920,12 +1097,14 @@ class DomainObject(DomainBase):
             del(self.registerCache[register])
 
     def createRegister(self, **kwds):
+        #self.logger.debug("Model: Creating register (in domain object).")
         if not ('typeName' in kwds and kwds['typeName']):
             kwds['typeName'] = self.__name__
         if not ('keyName' in kwds and kwds['keyName']):
             kwds['keyName']  = self.registerKeyName
         if not ('isCached' in kwds and kwds['isCached']):
-            kwds['isCached'] = self.isConstant
+            kwds['isCached'] = self.meta.isCached
+        #self.log.debug("Register for %s isCached = %s" % (self.__name__, kwds['isCached']))
         if not('searchAttributeNames' in kwds and kwds['searchAttributeNames']):
             kwds['searchAttributeNames'] = self.searchAttributeNames
         if not('startsWithAttributeName' in kwds and kwds['startsWithAttributeName']):
@@ -967,15 +1146,18 @@ class DomainObject(DomainBase):
         return self.getLabelValue()
 
     def createCollectiveNoun(self):
-        noun = self.createNoun()
-        # caution: naive pluralisation ahead!
-        if noun[-1] == 'y':
-            noun = noun[:-1] + 'ies'
-        elif noun[-1] == 's':
-            noun = noun[:-1] + 'ses'
-        else:
-            noun = noun + 's'
-        return noun
+        if not self.collectiveNoun:
+            noun = self.createNoun()
+            # Caution: naive pluralisation ahead!
+            if noun[-1] == 'y':
+                self.collectiveNoun = noun[:-1] + 'ies'
+            elif noun[-1] == 's':
+                self.collectiveNoun = noun + 'es'
+            elif noun[-2:] == 'ch':
+                self.collectiveNoun = noun + 'es'
+            else:
+                self.collectiveNoun = noun + 's'
+        return self.collectiveNoun
 
     createCollectiveNoun = classmethod(createCollectiveNoun)
     
@@ -991,13 +1173,21 @@ class DomainObject(DomainBase):
         register = metaAttr.getAssociatedObjectRegister(attrValue)
         return register
 
+        
+# Todo: Create 'dm.dom.main' with Simple, Named, Dated, and Standard, and with Stateful remaining in dm.dom.stateful?
+
 
 class SimpleObject(DomainObject):
+
     pass
+
     
-    
-class SimpleNamedObject(DomainObject):
+class NamedObject(DomainObject):
+
+    name = String(isIndexed=True, isRequired=True, isUnique=True, regex='')
 
     registerKeyName = 'name'
-    name = String(isIndexed=True, isRequired=True)
+    sortCaseInsensitive = True
+    sortOnName = 'name'
+    startsWithAttributeName = 'name'
 
