@@ -21,26 +21,29 @@ import os
 import inspect
 from dm.exceptions import *
 moddebug = False
-
-
+import traceback
+import weakref
 
 # todo: Add Email attribute meta class.
 
+class AttributeValidator(object):
+
+    def __init__(self, metaAttr, objectData, domainObject, objectRegister):
+        self.metaAttr = metaAttr
+        self.objectData = objectData
+        self.domainObject = domainObject
+        self.objectRegister = objectRegister
+        self.validate()
+
+    def validate(self):
+        raise Exception, "Method not implemented on %s" % self.__class__
+
+
 class NotDefined(object):
     "Internal 'null' value."
     pass
 
 
-# Todo: Somthing about this being used *only* in the db layer (now).
-class MetaBase(object):
-    "Domain model meta supertype."
-
-    registry = RequiredFeature('DomainRegistry')
-    dictionary = RequiredFeature('SystemDictionary')
-    logger = RequiredFeature('Logger')
-    debug = RequiredFeature('Debug')
-
-
 class BaseDomainMeta(object):
     "Domain model meta supertype."
 
@@ -49,7 +52,7 @@ class BaseDomainMeta(object):
     logger = RequiredFeature('Logger')
     debug = RequiredFeature('Debug')
 
-    def resolvePathPart(self, pathPart):
+    def resolvePathPart(self, pathPart, **kwds):
         return pathPart
 
     def calcTitle(self):
@@ -118,21 +121,22 @@ class MetaDomainObject(BaseDomainMeta):
 
     def __setattr__(self, attrName, attrVal):
         if issubclass(attrVal.__class__, MetaDomainAttr):
-            #if attrName in self.reservedAttrNames:
             if attrName in self.getReservedAttrNames():
                 msg = "Domain attribute name '%s' is reserved." % attrName
                 raise Exception(msg)
-            if not attrName in self.attributeNames:
-                attrVal.name = attrName
-                self.attributes.append(attrVal)
-                self.attributeNames[attrName] = attrVal
+            metaAttr = attrVal
+            if attrName in self.attributeNames:
+                msg = "Domain attribute '%s' somehow already set on '%s' meta object. Attribute names: %s" % (attrName, self.name, self.attributeNames.keys())
+                raise Exception(msg)
+            metaAttr.setName(attrName)
+            self.attributes.append(metaAttr)
+            self.attributeNames[attrName] = metaAttr
         else:
             super(MetaDomainObject, self).__setattr__(attrName, attrVal)
 
     def getReservedAttrNames(self):
         from dm.dom.base import DomainObject
         return [i[0] for i in inspect.getmembers(DomainObject)]
-        #return DomainObject.__dict__.keys()
 
     def createDomainClass(self, baseClass):
         className = self.name
@@ -164,10 +168,10 @@ class MetaDomainAttr(BaseDomainMeta):
     isAssociateList = False
     isImageFile = False
 
-    def __init__(self, typeName='', name='', dbName='', default=NotDefined, title='', comment='', isEditable=True, isHidden=False, isRequired=True, getChoices=None, isTemporal=False, isBitemporal=False, isIndexed=False, **kwds):
+    def __init__(self, typeName='', name='', dbName='', default=NotDefined, title='', comment='', isHidden=False, isEditable=True, isRequired=True, isSystem=False, getChoices=None, isTemporal=False, isBitemporal=False, isIndexed=False, isUnique=False, isImmutable=False, validators=[], **kwds):
         self.typeName = typeName
-        self.name = name
         self.dbName = dbName or name
+        self.setName(name)
         if default != NotDefined:
             self.default = default
         self.title = title        # presentable name
@@ -175,11 +179,14 @@ class MetaDomainAttr(BaseDomainMeta):
         #    self.title = self.spaceFromCamel(self.name)
         #self.title = self.spaceFromCamel(self.name)
         self.comment = comment
-        self.isEditable = isEditable # Present it, but don't allow edits.
-        self.isHidden = isHidden     # Don't present it.
+        self.isHidden = isHidden     # Don't present it (users unable to: init, read, update).
+        self.isEditable = isEditable # Allow edits (users can update).
         if self.isHidden:
             self.isEditable = False
-        self.isRequired = isRequired # Require value.
+        self.isRequired = isRequired # Require a value.
+        self.isSystem = isSystem     # Value controlled by system, not even admins can edit...
+        if self.isSystem:
+            self.isEditable = False  # ...so users also can't edit.
         self.getChoices = getChoices
         self.isIndexed = isIndexed
         self.isTemporal = isTemporal
@@ -189,6 +196,14 @@ class MetaDomainAttr(BaseDomainMeta):
         if self.isBitemporal:
             self.isTemporal = True
         self.isBitemporalActual = False
+        self.isUnique = isUnique
+        self.isImmutable = isImmutable
+        self.validators = validators
+
+    def setName(self, name):
+        self.name = name
+        if not self.dbName:
+            self.dbName = name
 
     def __repr__(self):
         return "<%s typeName='%s' name='%s'>" % (self.__class__.__name__, self.typeName, self.name)
@@ -196,9 +211,8 @@ class MetaDomainAttr(BaseDomainMeta):
     def isValueObject(self):
         return False
 
-    # Todo: Remove; apparently redundant given isAssociateList attribute?.
     def isList(self):
-        return False
+        return self.isAssociateList
 
     def isPaged(self):
         return False
@@ -230,7 +244,7 @@ class MetaDomainAttr(BaseDomainMeta):
     def createLabelValue(self, domainObject):
         return unicode(getattr(domainObject, self.name))
     # Todo: Replace createValueRepr() with createLabelValue() everywhere.
-    def createValueRepr(self, domainObject):
+    def createValueRepr(self, domainObject, absoluteUriPrefix=None):
         return self.createLabelValue(domainObject)
 
     def createLabelRepr(self, domainObject):
@@ -254,7 +268,7 @@ class MetaDomainAttr(BaseDomainMeta):
             self.setAttributeValue(domainObject, attrValue)
 
     def makeValueFromMultiValueDict(self, multiValueDict):
-        raise Exception, "Abstract method not implemented."
+        raise Exception, "Abstract method not implemented on '%s'." % self.__class__
 
     def setAttributeValue(self, domainObject, attrValue):
         setattr(domainObject, self.name, attrValue)
@@ -297,6 +311,27 @@ class MetaDomainAttr(BaseDomainMeta):
     def duplicateSelf(self):
         return type(self)()
 
+    def validateValue(self, objectData, domainObject, objectRegister):
+        self.validateUniqueness(objectData, domainObject, objectRegister)
+        for validator in self.validators:
+            validator(self, objectData, domainObject, objectRegister)
+
+    def validateUniqueness(self, objectData, domainObject, objectRegister):
+        if self.isUnique and self.name in objectData:
+            attrValue = self.makeValueFromMultiValueDict(objectData)
+            #attrValue = objectData[self.name]
+            kwds = { self.name: attrValue }
+            # It's either an update with an unchanged value.
+            if domainObject and getattr(domainObject, self.name) == attrValue:
+                pass
+            # Otherwise, an existing object indicates a conflict.
+            elif objectRegister.findDomainObjects(**kwds):
+                msg = self.formatUniquenessErrorMsg(attrValue)
+                raise ValidationError(msg)
+
+    def formatUniquenessErrorMsg(self, attrValue):
+        return "The %s '%s' is already being used." % (self.name, attrValue)
+
 
 class ValueObjectAttr(MetaDomainAttr):
     "Models a domain object value attribute."
@@ -337,7 +372,10 @@ class String(ValueObjectAttr):
 
     isString = True
 
-    def __init__(self, default='', **kwds):
+    def __init__(self, default='', regex=None, minLength=None, maxLength=None, **kwds):
+        self.regex = regex
+        self.minLength = minLength
+        self.maxLength = maxLength
         super(String, self).__init__(default=default, **kwds)
 
 
@@ -375,7 +413,7 @@ class Password(String):
     def __init__(self, isRequired=False, **kwds):
         super(Password, self).__init__(isRequired=isRequired, **kwds)
 
-    def createValueRepr(self, domainObject):
+    def createValueRepr(self, domainObject, absoluteUriPrefix=None):
         # Don't represent the password digest...
         if self.dictionary[DB_MIGRATION_IN_PROGRESS]:
             # ...unless data migration is in progress, when it's required.
@@ -432,7 +470,7 @@ class DateTime(ValueObjectAttr):
             self.logger.debug(msg)
         return attrValue
 
-    def createValueRepr(self, domainObject):
+    def createValueRepr(self, domainObject, absoluteUriPrefix=None):
         dateTimeObject = getattr(domainObject, self.name)
         dateTimeString = self.convertor.toHTML(dateTimeObject)
         return dateTimeString
@@ -498,7 +536,7 @@ class Boolean(ValueObjectAttr):
                 self.logger.debug(msg)
         return attrValue
 
-    def createValueRepr(self, domainObject):
+    def createValueRepr(self, domainObject, absoluteUriPrefix=None):
         attrValue = getattr(domainObject, self.name)
         if attrValue:
             return u'on'
@@ -518,10 +556,12 @@ class NumberObjectAttr(ValueObjectAttr):
     isNumber = True
     numberType = None
 
-    def __init__(self, default=0, **kwds):
+    def __init__(self, default=0, minValue=None, maxValue=None, **kwds):
+        self.minValue = minValue
+        self.maxValue = maxValue
         super(NumberObjectAttr, self).__init__(default=default, **kwds)
 
-    def resolvePathPart(self, pathPart):
+    def resolvePathPart(self, pathPart, **kwds):
         return self.numberType(pathPart)
 
     def makeValueFromMultiValueDict(self, multiValueDict):
@@ -537,7 +577,7 @@ class NumberObjectAttr(ValueObjectAttr):
             self.logger.debug(msg)
         return attrValue
 
-    def createValueRepr(self, domainObject):
+    def createValueRepr(self, domainObject, absoluteUriPrefix=None):
         return getattr(domainObject, self.name)
 
     def createLabelRepr(self, domainObject):
@@ -620,28 +660,30 @@ class ImageFile(MetaDomainAttr):
 class DomainObjectAssociation(MetaDomainAttr):
     "Associates domain objects with other domain objects."
 
-    def countChoices(self, domainObject):
+    def getAllChoices(self, objectRegister, domainObject):
         if callable(self.getChoices):
-            return len(self.getChoices(domainObject))
+            choices = self.getChoices(objectRegister, domainObject)
         else:
-            return len(self.getAssociatedObjectRegister(domainObject))
-
-    def getAllChoices(self, domainObject):
-        if callable(self.getChoices):
-            return self.getChoices(domainObject)
-        choices = []
-        try:
-            associateRegister = self.getAssociatedObjectRegister(domainObject)
-            for associateObject in associateRegister.getSortedList():
-                key = associateObject.getRegisterKeyValue()
-                label = associateObject.getLabelValue()
-                choice = (key, label)
-                choices.append(choice)
-        except Exception, inst:
-            message = "Couldn't get choices for %s: %s" % (
-               self.typeName, unicode(inst)
-            )
-            raise Exception(message)
+            choices = []
+            try:
+                associateRegister = self.getAssociatedObjectRegister(domainObject)
+                objectList = []
+                if associateRegister != []:
+                    objectList = associateRegister.getSortedList()
+                for associateObject in objectList:
+                    uri = associateObject.getUri()
+                    label = associateObject.getLabelValue()
+                    choice = (uri, label)
+                    choices.append(choice)
+                return choices
+            except Exception, inst:
+                trace = traceback.format_exc()
+                self.logger.error("Error: %s: %s" % (inst, trace))
+                message = "Couldn't get choices for %s: %s" % (
+                   self.typeName, unicode(inst)
+                )
+                raise Exception(message)
+        self.logger.debug("Choices for '%s': %s" % (self.name, choices))
         return choices
 
     def getAssociatedObjectRegister(self, domainObject):
@@ -654,16 +696,16 @@ class DomainObjectAssociation(MetaDomainAttr):
     def getDomainClass(self):
         return self.registry.getDomainClass(self.typeName)
         
-    def getObjectByKey(self, key):
+    def getObjectByKey(self, key, **kwds):
         domainRegister = self.getDomainRegister()
         if self.dictionary[DB_MIGRATION_IN_PROGRESS]:
             # Need all objects when migrating.
             if domainRegister.isStateful:
                 domainRegister = domainRegister.getAll()
-        return domainRegister[key]
+        return domainRegister.find(key, **kwds)
 
-    def resolvePathPart(self, pathPart):
-        return self.getObjectByKey(pathPart)
+    def resolvePathPart(self, pathPart, **kwds):
+        return self.getObjectByKey(pathPart, **kwds)
 
 
 class DomainObjectRef(DomainObjectAssociation):
@@ -677,11 +719,14 @@ class DomainObjectRef(DomainObjectAssociation):
             raise Exception, "No className for DomainObjectRef instance."
         super(DomainObjectRef, self).__init__(typeName=className, **kwds)
     
+    def formatUniquenessErrorMsg(self, attrValue):
+        return "There is already one for '%s'." % attrValue.getLabelValue()
+
     def getAquaintance(self, domainObject):
         return getattr(domainObject, self.name)
         
     def getRegistryAttrName(self, domainObject):
-        aquintanceClass = self.registry.getDomainClass(self.typeName)
+        aquintanceClass = self.getDomainClass()
         return aquintanceClass.getRegistryAttrName()
 
     def createInitialValue(self, domainObject):
@@ -690,10 +735,10 @@ class DomainObjectRef(DomainObjectAssociation):
                 return self.getObjectByKey(self.default)
         return None
 
-    def createValueRepr(self, domainObject):
+    def createValueRepr(self, domainObject, absoluteUriPrefix=None):
         value = getattr(domainObject, self.name)
         if value:
-            return value.getRegisterKeyValue()
+            return value.getUri(absoluteUriPrefix=absoluteUriPrefix)
         else:
             return value
        
@@ -709,7 +754,10 @@ class DomainObjectRef(DomainObjectAssociation):
     def makeValueFromMultiValueDict(self, multiValueDict):
         refValue = multiValueDict[self.name]
         if refValue:
-            attrValue = self.getObjectByKey(refValue)
+            if self.dictionary[DB_MIGRATION_IN_PROGRESS] and (isinstance(refValue, int) or (isinstance(refValue, basestring) and refValue[0] != '/')):
+                attrValue = self.getObjectByKey(refValue)
+            else:
+                attrValue = self.registry.dereference(refValue).target
         elif hasattr(self, 'default'):
             attrValue = self.getObjectByKey(self.default)
         else:
@@ -838,67 +886,54 @@ class AssociateList(DomainObjectAssociation):
     isPagedList = False
     registerClass = None
 
-    def __init__(self, className='', key='', owner='', isRequired=False, **kwds):
+    def __init__(self, className='', key='', owner='', ownerAsOwnerAttr='', isRequired=False, **kwds):
         super(AssociateList, self).__init__(typeName=className, isRequired=isRequired, **kwds)
-        self.key = key
-        self.owner = owner
+        self.key = key # Todo: Rename to keyName.
+        # Register key name defaults to the domainClass registerKeyName attribute.
+        self.owner = owner # Todo: Rename to ownerName.
+        self.ownerAsOwnerAttr = ownerAsOwnerAttr # Todo: Rename.
    
-    # Todo: Remove; apparently redundant given isAssociateList attribute?.
-    def isList(self):
-        return True
-
-    def countChoices(self, domainObject):
-        associateRegister = self.getAssociatedObjectRegister(domainObject)
-        return len(associateRegister)
-
-    def getAllChoices(self, domainObject):
-        choices = []
-        associateRegister = self.getAssociatedObjectRegister(domainObject)
-        objectList = []
-        if associateRegister != []:
-            objectList = associateRegister.getSortedList()
-        for associateObject in objectList:
-            key = associateRegister.getRegisterKey(associateObject)
-            if self.isKeyDomainObject():
-                label = associateObject.getLabelValue()
-            else:
-                label = associateObject.getLabelValue()  # Trying to get 
-                # ... iteration label values in concern update view. Was:
-                # label = key 
-            choice = (key, label)
-            choices.append(choice)
-        return choices
+    def getKey(self):
+        if not self.key:
+            domainClass = self.getDomainClass()
+            self.key = domainClass.registerKeyName
+        return self.key
 
     def createInitialValue(self, domainObject):
         # Just instantiate a register for this attribute on this object.
-        return self.createRegister(domainObject)
+        if type(domainObject).meta.isCached:
+            return self.createRegister(domainObject)
+        return NotDefined
+
+#    def setInitialValue(self, domainObject):
+#        pass
 
     def createRegister(self, domainObject):
-        recordClass = self.registry.getDomainClass(self.typeName)
-        registerClass = recordClass.registerClass
+        #self.logger.info("Model: Creating %s register (in meta) with owner: %s" % (self.typeName, domainObject))
+        #self.logger.info("Model: Creating %s register (in meta) with owner." % (self.typeName))
+        domainClass = self.registry.getDomainClass(self.typeName)
+        registerClass = domainClass.registerClass
         if self.isTemporal:
             className = "Temporal%s" % registerClass.__name__
             classBases = (TemporalRegisterMixin, registerClass)
             registerClass = type(className, classBases, {})
         register = registerClass(
             typeName=self.typeName,
-            keyName=self.key,
+            keyName=self.getKey(),
             ownerName=self.owner,
             owner=domainObject,
-            metaAttr=self
+            ownerAsOwnerAttrName=self.ownerAsOwnerAttr,
+            metaAttr=self,
+            isCached=domainClass.meta.isCached,
         )
         return register
 
-    def createValueRepr(self, domainObject):
-        valuesList = []
+    def createValueRepr(self, domainObject, absoluteUriPrefix=None):
         register = self.getAssociationObjectRegister(domainObject)
-        if register == None:
-            return valuesList
-        if self.isKeyDomainObject():
-            valuesList = [unicode(i.getRegisterKeyValue()) for i in register.keys()]
+        if self.getDomainClass().isImplicitAssociation:
+            return register.getUris(absoluteUriPrefix)
         else:
-            valuesList = [unicode(i) for i in register.keys()]
-        return valuesList
+            return register.getUri(absoluteUriPrefix)
 
     def createLabelRepr(self, domainObject):
         register = self.getAssociationObjectRegister(domainObject)
@@ -962,61 +997,77 @@ class AssociateList(DomainObjectAssociation):
     def setAttributeFromMultiValueDict(self, domainObject, multiValueDict):
         if hasattr(multiValueDict, 'getlist'):
             # It's a MultiValueDict (Django 0.96).
-            associatedObjectKeys = multiValueDict.getlist(self.name)
+            associatedObjectUris = multiValueDict.getlist(self.name)
         else:
             # It's just a dict (Django 1.0).
-            associatedObjectKeys = multiValueDict[self.name]
-        associatedObjectRegister = self.getAssociatedObjectRegister(
-            domainObject)
-        associationObjectRegister = self.getAssociationObjectRegister(
-            domainObject)
-        # delete where surplus
+            associatedObjectUris = multiValueDict[self.name]
+        associatedObjectRegister = self.getAssociatedObjectRegister(domainObject) # e.g. licenses for a project
+        associationObjectRegister = self.getAssociationObjectRegister(domainObject) # e.g. projectlicense "join table"
+        # Delete where surplus.
         for associationObject in associationObjectRegister:
-            associatedObjectKey = getattr(associationObject, self.key)
             if self.isKeyDomainObject():
-                associatedObjectKey = associatedObjectKey.getRegisterKeyValue()
-            if not unicode(associatedObjectKey) in associatedObjectKeys:
+                associatedObject = getattr(associationObject, self.getKey())
+                associatedObjectUri = associatedObject.getUri()
+            else:
+                raise Exception, "Temporarily not supported."
+            if not unicode(associatedObjectUri) in associatedObjectUris:
                 associationObject.delete()
-        # create where missing
-        for associatedObjectKey in associatedObjectKeys:
-            if associatedObjectKey == None:
+        # Create where missing.
+        for associatedObjectUri in associatedObjectUris:
+            if associatedObjectUri == None:
                 continue
             if self.isKeyDomainObject():
-                associatedObjectKey = associatedObjectRegister[associatedObjectKey]
-            if not associatedObjectKey in associationObjectRegister:
-                associationObjectRegister.create(associatedObjectKey)
+                associatedObject = self.registry.dereference(associatedObjectUri).target
+                if associatedObject and associatedObject not in associationObjectRegister:
+                    associationObjectRegister.create(associatedObject)
+            else:
+                raise Exception, "Temporarily not supported."
 
-    def getAssociationObjectRegister(self, domainObject):
-        return getattr(domainObject, self.name)
-        
     def isKeyDomainObject(self):
-        if self.key == 'id':
+        if self.getKey() == 'id':
             return False
         keyAttr = self.getKeyMetaAttribute()
         return keyAttr.isDomainObjectRef
         
     def getKeyMetaAttribute(self):
         domainClass = self.getDomainClass()
-        if self.key in domainClass.meta.attributeNames:
-            return domainClass.meta.attributeNames[self.key]
+        if self.getKey() in domainClass.meta.attributeNames:
+            return domainClass.meta.attributeNames[self.getKey()]
         else:
             msg = "For class '%s', key '%s' not in attributes: %s." % (
                 domainClass.meta.name,
-                self.key,
+                self.getKey(),
                 domainClass.meta.attributeNames.keys(),
             )
             raise Exception(msg)
         
+    def getAssociationObject(self, domainObject, keyValue):
+        register = self.getAssociationObjectRegister(domainObject)
+        keyValue = self.getKeyValue(keyValue)
+        return register[keyValue]
+
+    def getAssociationObjectRegister(self, domainObject):
+        return getattr(domainObject, self.name)
+        
+    def getKeyValue(self, domainObject, keyValue):
+        # If the key is a domain object, then the key value string needs to be
+        # converted into a key value domain object.
+        if self.isKeyDomainObject():
+            keyMeta = self.getKeyMetaAttribute()
+            associateClassName = keyMeta.typeName
+            associateClass = self.registry.getDomainClass(associateClassName)
+            associateRegister = associateClass.createRegister()
+            keyValue = associateRegister[keyValue]
+        return keyValue
+
     def getAssociatedObjectRegister(self, domainObject):
+        # If the key is a domain object, then it is assumed that the
+        # register keys are the associated objects in a many-many relationship,
+        # and the association objects are being used to model to associations.
+        # Otherwise the association objects are also the associated objects, 
+        # and the association is modelled as an attribute on associated object.
+        # Todo: Make this explicit somehow.
         register = None
-        #
-        # Note well:
-        #
-        # Implication that if they key is a domain object the focus is on the
-        # keys and the register objects are many-many association objects.
-        #
-        # Todo: Make this explicit somehow, like with isKeyAssociate() rather
-        #       than the implicit isKeyDomainObject inference.
         if self.isKeyDomainObject():
             keyMeta = self.getKeyMetaAttribute()
             associateClassName = keyMeta.typeName
@@ -1029,16 +1080,23 @@ class AssociateList(DomainObjectAssociation):
                 register = []
         return register   
 
-    def getAssociationObject(self, domainObject, keyValue):
-        register = self.getAssociationObjectRegister(domainObject)
-        if self.isKeyDomainObject():
-            keyMeta = self.getKeyMetaAttribute()
-            associateClassName = keyMeta.typeName
-            associateClass = self.registry.getDomainClass(associateClassName)
-            associateRegister = associateClass.createRegister()
-            keyValue = associateRegister[keyValue]
-        associationObject = register[keyValue]
-        return associationObject
+    def makeValueFromMultiValueDict(self, multiValueDict):
+        if self.name not in multiValueDict:
+            attrValue = []
+        else:
+            refValue = multiValueDict[self.name]
+            raise Exception, "Need to finish this?"
+        return attrValue
+        #if refValue:
+        #    attrValue = self.getObjectByKey(refValue)
+        #elif hasattr(self, 'default'):
+        #    attrValue = self.getObjectByKey(self.default)
+        #else:
+        #    attrValue = None
+        #if moddebug and self.debug:
+        #    msg = "Made %s value: %s" % (self.name, attrValue)
+        #    self.logger.debug(msg)
+        #return attrValue
 
 
 class HasA(Associate):
