@@ -16,17 +16,17 @@ class TestPerson(TestCase):
     def setUp(self):
         super(TestPerson, self).setUp()
         self.fixtureName = 'TestPerson'
-        self.persons = self.registry.persons
+        self.people = self.registry.people
         try:
-            person = self.persons.getAll()[self.fixtureName]
+            person = self.people.getAll()[self.fixtureName]
             person.delete()
             person.purge()
         except:
-            self.person = self.persons.create(self.fixtureName)
+            self.person = self.people.create(self.fixtureName)
 
     def tearDown(self):
         try:
-            person = self.persons.getAll()[self.fixtureName]
+            person = self.people.getAll()[self.fixtureName]
             person.delete()
             person.purge()
         except:
@@ -34,8 +34,16 @@ class TestPerson(TestCase):
         self.person = None
 
     def test_register(self):
-        self.failUnless(self.persons.requiredState)
-        self.failUnless(self.persons.requiredState.record)
+        self.failUnless(self.people.requiredState)
+        self.failUnless(self.people.requiredState.record)
+        # Check regex on name meta attribute.
+        meta = self.people.getDomainClassMeta()
+        self.failUnless(meta.attributeNames['name'].regex)
+        # Check getUris().
+        self.failUnlessEqual(self.people.getUri('/x'), '/x/people')
+        # Check getUri().
+        expected = ['/x/people/'+i.getRegisterKeyValue() for i in self.people]
+        self.failUnlessEqual(self.people.getUris('/x'), expected)
    
     def test_new(self):
         self.failUnless(self.person, "New person could not be created.")
@@ -46,15 +54,17 @@ class TestPerson(TestCase):
             "No role for new person."
         )
         self.failUnlessRaises(KforgeDomError,
-            self.registry.persons.create, self.fixtureName
+            self.registry.people.create, self.fixtureName
         )
+        # Check getUri().
+        self.failUnlessEqual(self.person.getUri('/x'), '/x/people/%s' % self.fixtureName)
 
     def test_find(self):
-        self.failUnless(self.registry.persons['TestPerson'],
+        self.failUnless(self.registry.people['TestPerson'],
             "New person could not be found."
         )
         self.failUnlessRaises(KforgeRegistryKeyError,
-            self.registry.persons.__getitem__, 'TestAlien'
+            self.registry.people.__getitem__, 'TestAlien'
         )
 
     def test_delete(self):
@@ -75,7 +85,7 @@ class TestPerson(TestCase):
         )
 
     def test_purge(self):
-        self.failUnless(self.registry.persons['TestPerson'],
+        self.failUnless(self.registry.people['TestPerson'],
             "Purged active person could not be found."
         )
         self.person.delete()
@@ -84,20 +94,20 @@ class TestPerson(TestCase):
         )
         self.person.purge()
         self.failUnlessRaises(KforgeRegistryKeyError,
-            self.registry.persons.__getitem__, self.fixtureName
+            self.registry.people.__getitem__, self.fixtureName
         )
 
     def test___delitem__(self):
-        del self.persons[self.fixtureName]
+        del self.people[self.fixtureName]
         self.failUnlessRaises(KforgeRegistryKeyError,
-            self.registry.persons.__getitem__, self.fixtureName
+            self.registry.people.__getitem__, self.fixtureName
         )
 
     def test_is(self):
-        self.failUnless(self.persons.has_key(self.fixtureName),
+        self.failUnless(self.people.has_key(self.fixtureName),
             "New person doesn't appear to be there."
         )
-        self.failIf(self.persons.has_key('TestAlien'),
+        self.failIf(self.people.has_key('TestAlien'),
             "Strange person does appear to be there."
         )
         self.failUnless(self.person.state,
@@ -108,7 +118,7 @@ class TestPerson(TestCase):
         )
         self.person.delete()
         self.person.purge()
-        self.failIf(self.persons.has_key(self.fixtureName),
+        self.failIf(self.people.has_key(self.fixtureName),
             "New person still appears to be there."
         )
 
@@ -119,7 +129,7 @@ class TestPerson(TestCase):
             "Person doesn't have attribute."
         )
         self.person.save()
-        person = self.persons[self.fixtureName]
+        person = self.people[self.fixtureName]
         self.assertEquals(person.fullname, "Test Person",
             "Retrieved person has wrong fullname."
         )
@@ -129,36 +139,36 @@ class TestPerson(TestCase):
         )
         
     def test_count(self):
-        self.failUnless(self.persons.count(), "Problem with person count.")
+        self.failUnless(self.people.count(), "Problem with person count.")
 
     def test_keys(self):
-        self.failUnless(self.persons.keys(), "Problem with person list.")
+        self.failUnless(self.people.keys(), "Problem with person list.")
 
     def test_iter(self):
-        self.failUnless(self.persons.__iter__(), "Problem with person iter.")
-        for p in self.persons:
+        self.failUnless(self.people.__iter__(), "Problem with person iter.")
+        for p in self.people:
             self.failUnless(p.name, "Problem with iteration person: %s" % p)
 
     def test_getNextObject(self):
-        personList = self.persons.getSortedList()
+        personList = self.people.getSortedList()
         self.failUnless(len(personList) >= 4)
         person = personList[-3]
-        nextPerson = self.persons.getNextObject(personList, person)
+        nextPerson = self.people.getNextObject(personList, person)
         self.failUnlessEqual(nextPerson, personList[-2])
-        nextPerson = self.persons.getNextObject(personList, nextPerson)
+        nextPerson = self.people.getNextObject(personList, nextPerson)
         self.failUnlessEqual(nextPerson, personList[-1])
-        nextPerson = self.persons.getNextObject(personList, nextPerson)
+        nextPerson = self.people.getNextObject(personList, nextPerson)
         self.failIf(nextPerson)
         
     def test_getPreviousObject(self):
-        personList = self.persons.getSortedList()
+        personList = self.people.getSortedList()
         self.failUnless(len(personList) >= 4)
         person = personList[2]
-        previousPerson = self.persons.getPreviousObject(personList, person)
+        previousPerson = self.people.getPreviousObject(personList, person)
         self.failUnlessEqual(previousPerson, personList[1])
-        previousPerson = self.persons.getPreviousObject(personList, previousPerson)
+        previousPerson = self.people.getPreviousObject(personList, previousPerson)
         self.failUnlessEqual(previousPerson, personList[0])
-        previousPerson = self.persons.getPreviousObject(personList, previousPerson)
+        previousPerson = self.people.getPreviousObject(personList, previousPerson)
         self.failIf(previousPerson)
 
     def test_getOptionsRegister(self):
@@ -178,13 +188,13 @@ class TestPersonFilter(TestCase):
     def setUp(self):
         super(TestPersonFilter, self).setUp()
         self.fixtureName = 'TestPerson'
-        self.persons = self.registry.persons
-        allPersons = self.persons.getAll()
+        self.people = self.registry.people
+        allPersons = self.people.getAll()
         if self.fixtureName in allPersons:
             person = allPersons[self.fixtureName]
             person.delete()
             person.purge()
-        self.person = self.persons.create(self.fixtureName)
+        self.person = self.people.create(self.fixtureName)
 
     def tearDown(self):
         if self.person:
@@ -193,10 +203,10 @@ class TestPersonFilter(TestCase):
         super(TestPersonFilter, self).tearDown()
 
     def test_filter(self):
-        self.persons.filter = {'name': self.fixtureName}
-        self.failUnlessEqual(len(self.persons), 1)
-        self.persons.filter = {'name': 'NotaName'}
-        self.failUnlessEqual(len(self.persons), 0)
-        self.persons.filter = {}
+        self.people.filter = {'name': self.fixtureName}
+        self.failUnlessEqual(len(self.people), 1)
+        self.people.filter = {'name': 'NotaName'}
+        self.failUnlessEqual(len(self.people), 0)
+        self.people.filter = {}
    
 
