@@ -1,14 +1,38 @@
 from dm.dom.stateful import *
 from dm.ioc import *
+from dm.dictionarywords import PLUGINS_AVAILABLE
+from dm.exceptions import ValidationError
 
 def getProjects():
     domainRegistry = RequiredFeature('DomainRegistry')
     return domainRegistry.projects
 
+def getPluginChoices():
+    dictionary = RequiredFeature('SystemDictionary')
+    availablePlugins = PLUGINS_AVAILABLE.parse(dictionary[PLUGINS_AVAILABLE])
+    return [(i, i) for i in availablePlugins]
+
+
+class PluginNameValidator(AttributeValidator):
+
+    pluginFactory = RequiredFeature('PluginFactory')
+
+    def validate(self):
+        attrName = self.metaAttr.name
+        if attrName in self.objectData:
+            attrValue = self.objectData[attrName]
+            if not self.pluginFactory.getPluginClass(attrValue):
+                msg = "Cannot load '%s' plugin." % attrValue
+                raise ValidationError(msg)
+
+
 class Plugin(StandardObject):
     "Registered plugin."
 
-    systemFactory = RequiredFeature('PluginFactory')
+    name = String(isIndexed=True, isRequired=True, isImmutable=True,
+        isUnique=True, validators=[PluginNameValidator], getChoices=getPluginChoices)
+
+    pluginFactory = RequiredFeature('PluginFactory')
    
     def __init__(self, **kwds):
         super(Plugin, self).__init__(**kwds)
@@ -24,12 +48,12 @@ class Plugin(StandardObject):
     def getSystem(self):
         "Returns plugin system modelled by domain object."
         if not self.__system:
-            self.__system = self.systemFactory.getPlugin(self)
+            self.__system = self.pluginFactory.getPlugin(self)
         return self.__system
 
-    def extendsDomainModel(self):
-        "Whether or not the plugin system has it's own domain objects."
-        return self.getSystem().extendsDomainModel
+    def getModelExtnClass(self):
+        "Returns the model extention domain class for this plugin."
+        return self.getSystem().getModelExtnClass()
 
     def getExtnRegister(self):
         "Returns the plugin system's domain object register."
