@@ -10,27 +10,31 @@ from dm.ioc import *
 from dm.exceptions import * 
 import inspect
 import dm.times
-
+import weakref
+import os
 
 class DomainRegistry(AbstractRegister):
-    "Holds top-level domain object registers."
+    """
+    Central register for the domain model classes and top-level model registers.
+    """
 
     def __init__(self):
-        "Initialises top-level domain object registers."
         super(DomainRegistry, self).__init__()
         self.domainClassRegister = None
         self.backgroundObjectCache = []
         self.path = ModelPath(self)
 
     def registerDomainClass(self, domainClass):
-        self.log.debug("Registering domain class: %s" % domainClass)
-        baseClass = DomainObject
-        if not issubclass(domainClass, baseClass):
-            raise Exception, "Class is not subclass of DomainObject."
-        domainClassName = domainClass.__name__
+        """
+        Activates given domain class with the domain model.
+        """
+        self.log.debug("Registering domain class '%s'." % domainClass)
+        # Check domain class type.
+        if not issubclass(domainClass, DomainObject):
+            raise Exception, "Domain class %s is not subclass of DomainObject." % domainClass
+        # Check domain class name not already registered.
         classRegister = self.getDomainClassRegister()
-        #if domainClassName == 'TemporalObjectExampleHist_grants':
-        #    raise domainClassName
+        domainClassName = domainClass.__name__
         if domainClassName in classRegister:
             existingClass = classRegister[domainClassName]
             if features.allowReplace:
@@ -40,35 +44,57 @@ class DomainRegistry(AbstractRegister):
             )
             raise Exception, msg 
 
-        if not 'meta' in domainClass.__dict__ \
-        or not domainClass.__dict__['meta']:
+        # Create meta domain object for submitted domain class.
+        if not domainClass.__dict__.get('meta', None):
             domainClass.meta = domainClass.metaClass(domainClassName)
-            
         if not domainClass.meta.dbName:
             if domainClass.dbName:
                 domainClass.meta.dbName = domainClass.dbName
 
-        if domainClass.isTemporal==True:
+        # For temporal objects, infer temporal history attribute.
+        if domainClass.isTemporal:
             temporalClassName = self.makeTemporalRevisionClassName(domainClass.meta)
             domainClass.temporalHistory = HasMany(temporalClassName, 'id', 'parent', isHidden=True)
 
-        #
-        # 'Concrete Table Inheritance', Martin Fowler
-        # - http://martinfowler.com/eaaCatalog/concreteTableInheritance.html
-        #
-        self.setMetaAttributesFromClass(domainClass, domainClass.meta)
-        
+        # Set model attributes of domain class on meta domain object.
+        # Implements 'Concrete Table Inheritance' (Martin Fowler).
+        # http://martinfowler.com/eaaCatalog/concreteTableInheritance.html
+        self.setModelAttributesFromClass(domainClass, domainClass.meta)
+       
+        # Set meta model attributes of domain class on meta domain object.
         domainClass.meta.isUnique = domainClass.isUnique
-        domainClass.meta.isCached = domainClass.isConstant
+        isCached = False
+        if domainClass.isConstant == True:
+            isCached = True
+        elif bool(self.dictionary[MODEL_CACHE_IS_ENABLED]):
+            cachedClasses = self.dictionary[MODEL_CACHE_CLASSES]
+            cachedClasses = cachedClasses.split(',')
+            cachedClasses = [i.strip() for i in cachedClasses if i.strip()]
+            if not cachedClasses:
+                isCached = True
+            elif domainClassName in cachedClasses:
+                isCached = True
+        self.log.debug("Domain class '%s' is %s in model in pid %s." % (domainClassName, isCached and 'cached' or 'NOT cached', os.getpid()))
+
+        domainClass.meta.isCached = isCached
         domainClass.meta.isTemporal = domainClass.isTemporal
         domainClass.meta.setSortOnAttr(domainClass.sortOnName)
+        domainClass.meta.sortCaseInsensitive = domainClass.sortCaseInsensitive
         domainClass.meta.sortAscending = domainClass.sortAscending
+
+        # Add domain class to register.
         classRegister[domainClassName] = domainClass
         domainClass.isRegistered = True
+
+        # Initialise reflection into the persistence layer.
         self.createPersistenceClass(domainClass.meta)
+
+        # Go ahead with any attributes that depend on the submitted class.
         self.setDeferredAttributesOnRegisteredClasses(domainClass)
         for name in classRegister:
             self.checkHasAsForHasManys(classRegister[name])
+
+        # For temporal objects, create temporal history domain class.
         if domainClass.isTemporal:
             # Group temporal propertes on revision object.
             metaAttrs = []
@@ -87,6 +113,19 @@ class DomainRegistry(AbstractRegister):
                         parentMeta = domainClass.meta
                         self.generateTemporalPropertyClass(parentMeta, attrMeta)
 
+        # Set domain class properties from meta attributes.
+        if not domainClass.meta.isCached:
+            self.setDomainClassProperties(domainClass)
+
+    def setDomainClassProperties(self, domainClass):
+        for attrMeta in domainClass.meta.attributes:
+            if attrMeta.isAssociateList:
+                self.setDomainClassProperty(domainClass, attrMeta)
+
+    def setDomainClassProperty(self, domainClass, attrMeta):
+        classProperty = property(attrMeta.createRegister)
+        setattr(domainClass, attrMeta.name, classProperty)
+
     def makeTemporalRevisionClassName(self, classMeta):
         return TemporalRevision.makeTemporalName(classMeta.name)
 
@@ -180,9 +219,6 @@ class DomainRegistry(AbstractRegister):
         "Creates and caches refs to all background domain objects."
         pass
         
-    def retrieveItem(self, key):
-        return getattr(self, key)
-
     def createDomainClass(self, metaDomainObject, baseClass=DomainObject):
         return metaDomainObject.createDomainClass(baseClass)
 
@@ -203,7 +239,7 @@ class DomainRegistry(AbstractRegister):
             raise Exception, "Domain class '%s' is not defined." % className
         return classRegister[className]
 
-    def setMetaAttributesFromClass(self, domainClass, domainClassMeta):
+    def setModelAttributesFromClass(self, domainClass, domainClassMeta):
         className = domainClass.__name__
         classAttrs = inspect.getmembers(domainClass)
         for (attrName, classAttr) in classAttrs:
@@ -214,7 +250,7 @@ class DomainRegistry(AbstractRegister):
                         raise Exception, "Usage of name '%s' for attribute HasA('%s') on '%s' class is not supported (the generated SQL confuses the database at the moment). Please use an attribute name that is case-insensitively distinct from the attribute's class name." % (attrName, className, className)
                 # fix up 'owner' for lists of associates
                 if issubclass(classAttr.__class__, AssociateList):
-                    if not classAttr.owner:
+                    if classAttr.owner == '':
                         classAttr.owner = defaultInstanceName
                 if issubclass(classAttr.__class__, HasA):
                     if self.isDomainClassRegistered(classAttr.typeName):
@@ -252,6 +288,8 @@ class DomainRegistry(AbstractRegister):
                                 newAttributes[attrName] = deferredAttr
                             elif issubclass(deferredAttr.__class__, HasMany):
                                 registeredClass.meta.__setattr__(attrName, deferredAttr)
+                                if not registeredClass.meta.isCached:
+                                    self.setDomainClassProperty(registeredClass, deferredAttr)
                                 newAttributes[attrName] = deferredAttr
             for attrName in newAttributes:
                 del(deferredAttrs[attrName])
@@ -268,14 +306,20 @@ class DomainRegistry(AbstractRegister):
                 classRegister = self.getDomainClassRegister()
                 if domainClassAttr.typeName in classRegister:
                     registeredClass = classRegister[domainClassAttr.typeName]
+                    if not domainClassAttr.owner:
+                        continue
+                    expectedClass = domainClass
+                    if domainClassAttr.ownerAsOwnerAttr: # Rename and comment.
+                        ownerAttr = domainClass.meta.attributeNames[domainClassAttr.ownerAsOwnerAttr]
+                        expectedClass = classRegister[ownerAttr.typeName]
                     if domainClassAttr.owner in registeredClass.meta.attributeNames:
                         hasA = registeredClass.meta.attributeNames[domainClassAttr.owner]
                         if not issubclass(hasA.__class__, HasA):
-                            raise Exception, "Mismatched domain object attribute definitions: HasMany('%s') on class '%s' expected a HasA('%s') called '%s', not '%s'." % (domainClassAttr.typeName, domainClass.__name__, domainClass.__name__, domainClassAttr.owner, hasA.__class__.__name__)
-                        if hasA.typeName != domainClass.__name__:
-                            raise Exception, "Mismatched domain object attribute definitions: HasMany('%s') on class '%s' expected a HasA('%s') called '%s', not HasA('%s')." % (domainClassAttr.typeName, domainClass.__name__, domainClass.__name__, domainClassAttr.owner, hasA.typeName)
+                            raise Exception, "Mismatched domain object attribute definitions: HasMany('%s') on class '%s' expected a HasA('%s') called '%s', not '%s'." % (domainClassAttr.typeName, domainClass.__name__, expectedClass.__name__, domainClassAttr.owner, hasA.__class__.__name__)
+                        if hasA.typeName != expectedClass.__name__:
+                            raise Exception, "Mismatched domain object attribute definitions: HasMany('%s') on class '%s' expected a HasA('%s') called '%s', not HasA('%s')." % (domainClassAttr.typeName, domainClass.__name__, expectedClass.__name__, domainClassAttr.owner, hasA.typeName)
                     else:
-                        raise Exception, "Missing domain object attribute definition: HasMany('%s') on class '%s' expected a HasA('%s') called '%s' on class '%s'. Attribute names are: %s" % (domainClassAttr.typeName, domainClass.__name__, domainClass.__name__, domainClassAttr.owner, domainClassAttr.typeName, registeredClass.meta.attributeNames.keys() )
+                        raise Exception, "Missing domain object attribute definition: HasMany('%s') on class '%s' expected a HasA('%s') called '%s' on class '%s'. Attribute names are: %s" % (domainClassAttr.typeName, domainClass.__name__, expectedClass.__name__, domainClassAttr.owner, domainClassAttr.typeName, registeredClass.meta.attributeNames.keys() )
         
     def addPersistenceAttribute(self, className, attribute):
         self.database.addPersistenceAttribute(className, attribute)
@@ -286,6 +330,15 @@ class DomainRegistry(AbstractRegister):
             object = register[key]
             self.backgroundObjectCache.append(object)
 
+    def dereference(self, path, **kwds):
+        """
+        Creates and returns a 'Dereference' instance for given path with 'target' and 'context' attributes.
+        """
+        return Dereference(self.registry, path, **kwds)
+
+    def retrieveItem(self, key):
+        return getattr(self, key)
+
 
 class RegistryPathGetter(object):
 
@@ -338,16 +391,30 @@ class RegistryPathGetter(object):
 class ModelPath(object):
 
     def __init__(self, modelRegistry):
-        self.registry = modelRegistry
+        self.registry = weakref.ref(modelRegistry)
 
     def open(self, registryPath):
         """
         Return model object for model path.
         """
-        modelObj = self.registry
+        modelObj = self.registry()
         for pathPart in registryPath.split('/'):
             if pathPart == '':
                 continue
             modelObj = modelObj.resolvePathPart(pathPart)
         return modelObj
 
+
+class Dereference(object):
+
+    def __init__(self, registry, path, **kwds):
+        self.target = registry
+        self.path = path
+        self.context = None
+        for name in self.path.split('/'):
+            if name == '':
+                continue
+            else:
+                self.context = self.target
+                self.target = self.target.resolvePathPart(name, **kwds)
+
