@@ -2,6 +2,7 @@ from dm.dom.base import *
 from dm.dom.meta import *
 import dm.exceptions  # todo: remove?
 import dm.times
+import datetime
 
 class StatefulRegister(DomainObjectRegister):
 
@@ -14,9 +15,22 @@ class StatefulRegister(DomainObjectRegister):
         self.initialiseWithStateName = 'active'
         self.requiredStateName = 'active'
 
+    def getUri(self, absoluteUriPrefix=None):
+        uri = super(StatefulRegister, self).getUri(absoluteUriPrefix)
+        if self.requiredStateName and self.requiredStateName != 'active':
+            uri += '/' + self.requiredStateName
+        return uri
+
+    def initialiseKwds(self, kwds):
+        super(StatefulRegister, self).initialiseKwds(kwds)
+        if 'state' not in kwds:
+            state = self.getInitialiseWithState()
+            if state:
+                kwds['state'] = state
+
     def coerceKwds(self, kwds):
         super(StatefulRegister, self).coerceKwds(kwds)
-        if not kwds.has_key('state'):
+        if 'state' not in kwds:
             state = self.getRequiredState()
             if state:
                 kwds['state'] = state
@@ -31,8 +45,7 @@ class StatefulRegister(DomainObjectRegister):
         if not self.requiredStateName:
             return None
         if not self.requiredState:
-            stateName = self.requiredStateName
-            self.requiredState = self.getState(stateName)
+            self.requiredState = self.getState(self.requiredStateName)
         return self.requiredState
 
     def getInitialiseWithState(self):
@@ -42,10 +55,9 @@ class StatefulRegister(DomainObjectRegister):
         return self.initialiseWithState
 
     def getState(self, name):
-        if name in self.registry.states:
-            return self.registry.states[name]
-        else:
+        if not name:
             return None
+        return self.registry.states[name]
 
 
 class CompoundRegister(StatefulRegister):
@@ -54,30 +66,32 @@ class CompoundRegister(StatefulRegister):
         super(CompoundRegister, self).__init__(typeName, **kwds)
         if isCompoundingRegister:
             self._initKwds = kwds
-            self._pending = None
-            self._deleted = None
-            self._all = None
-            
+
+    def getStateSpecificRegister(self, stateName):
+        name = "_%s" % stateName
+        value = hasattr(self, name) and getattr(self, name) or None
+        if value == None or value() == None:
+            register = self.createStateSpecificRegister(stateName)
+            setattr(self, name, weakref.ref(register))
+        else:
+            register = value()
+        return register
+
     def getPending(self):
-        if self._pending == None:
-            self._pending = self.createStateSpecificRegister('pending')
-        return self._pending
+        return self.getStateSpecificRegister('pending')
+    pending = property(getPending)
     
     def getDeleted(self):
-        if self._deleted == None:
-            self._deleted = self.createStateSpecificRegister('deleted')
-        return self._deleted
+        return self.getStateSpecificRegister('deleted')
+    deleted = property(getDeleted)
     
     def getAll(self):
-        if self._all == None:
-            self._all = self.createStateSpecificRegister('all')
-        return self._all
+        return self.getStateSpecificRegister('all')
+    all = property(getAll)
     
     def createStateSpecificRegister(self, stateName):
-        registerClass = self.__class__
-        kwds = self._initKwds
-        register = registerClass(
-            self.typeName, isCompoundingRegister=False, **kwds
+        register = type(self)(
+            self.typeName, isCompoundingRegister=False, **self._initKwds
         )
         if stateName == 'all':
             register.requiredStateName = ''
@@ -90,7 +104,7 @@ class CompoundRegister(StatefulRegister):
 class StatefulObject(DomainObject):
 
     registerClass = CompoundRegister
-    state = HasA('State', isEditable=False, default='active')
+    state = HasA('State', isSystem=True, isHidden=True) #, default='active')
 
     def __init__(self):
         super(StatefulObject, self).__init__()
@@ -123,22 +137,30 @@ class StatefulObject(DomainObject):
             self.state.purgeObject(self)
 
 
-class NamedStatefulObject(StatefulObject, SimpleNamedObject):
+class NamedStatefulObject(NamedObject, StatefulObject):
     pass
 
 
-class SimpleDatedObject(SimpleObject):
+class DatedObject(DomainObject):
 
     dateCreated = DateTime(
         default=dm.times.getUniversalNow,
         isIndexed=True,
-        isEditable=False
+        isSystem=True,
     )
     lastModified = DateTime(
         default=dm.times.getUniversalNow,
-        isEditable=False
+        isSystem=True,
     )
 
+    def saveSilently(self):
+        if not self.dictionary[DB_MIGRATION_IN_PROGRESS]:
+            self.updateLastModifiedTime()
+        super(DatedObject, self).saveSilently()
+
+    def updateLastModifiedTime(self):
+        self.lastModified = dm.times.getUniversalNow()
+        
     def dateCreatedLocal(self):
         # todo: convert to localtime
         return self.dateCreated
@@ -153,33 +175,28 @@ class SimpleDatedObject(SimpleObject):
     def getDeltaSinceLastModified(self):
         return dm.times.getUniversalNow() - self.lastModified
 
-    # Todo: Rename SystemUpSince to SystemStarted.
-    # Todo: Rename SYSTEM_UP_SINCE to SYSTEM_STARTED.
-    def isSystemUpSinceCreated(self):
-        systemStarted = self.dictionary[self.dictionary.words.SYSTEM_UP_SINCE]
-        return systemStarted > self.dateCreated
+    def isSystemStartedAfterDateCreated(self):
+        return self.getSystemStarted() > self.dateCreated
 
-    def isSystemUpSinceModified(self):
-        systemStarted = self.dictionary[self.dictionary.words.SYSTEM_UP_SINCE]
-        return systemStarted > self.lastModified
+    def isSystemStartedAfterLastModified(self):
+        return self.getSystemStarted() > self.lastModified
+
+    def getSystemStarted(self):
+        return self.dictionary[SYSTEM_STARTED]
 
 
-class SimpleNamedDatedObject(SimpleNamedObject, SimpleDatedObject):
+class NamedDatedObject(NamedObject, DatedObject):
     pass
 
 
-class DatedStatefulObject(StatefulObject, SimpleDatedObject):
+class DatedStatefulObject(DatedObject, StatefulObject):
     pass
 
 
-class NamedDatedStatefulObject(DatedStatefulObject, SimpleNamedObject):
+class StandardObject(NamedObject, DatedObject, StatefulObject):
     pass
 
 
-class StandardObject(NamedDatedStatefulObject):
-    pass
-    
-
 class PagedCompoundRegister(CompoundRegister):
 
     isPagedList = True
