@@ -1,12 +1,12 @@
-from dm.dom.stateful import SimpleDatedObject
+from dm.dom.stateful import DatedObject
 from dm.dom.stateful import String
 from dm.dictionarywords import SYSTEM_VERSION
 
-class System(SimpleDatedObject):
+class System(DatedObject):
     "Kforge installation."
 
     version = String(
-        default=SimpleDatedObject.dictionary[SYSTEM_VERSION]
+        default=DatedObject.dictionary[SYSTEM_VERSION], isSystem=True
     )
     mode = String(isRequired=False)
 
