@@ -1,7 +1,7 @@
 import unittest
 from dm.dom.testunit import TestCase
 from dm.exceptions import *
-from dm.dom.stateful import SimpleObject, SimpleNamedObject, NamedStatefulObject, StatefulObject, String, DateTime, HasA, HasMany, AggregatesMany
+from dm.dom.stateful import NamedObject, NamedStatefulObject, StatefulObject, String, DateTime, HasA, HasMany, AggregatesMany
 from dm.dom.base import DomainObjectRegister
 from dm.ioc import RequiredFeature
 import datetime
@@ -143,7 +143,7 @@ class TestTemporalObjectExample(TestCase):
 
         self.timepoint.recorded = revision3
         grants = [i for i in self.collection[''].grants]
-        self.failIf(grant1 in grants, grants)
+        self.failIf(grant1 in grants, (grant1, grants))
         self.failUnless(grant2 in grants, grants)
         self.failUnless(grant3 in grants, grants)
         self.failIf(grant4 in grants, (permission4, grants))
@@ -444,7 +444,7 @@ class TestBitemporalProperty(TestCase):
         self.failUnless(self.collection.getCreateTime() > time1)
 
 
-class Temporal(SimpleNamedObject):  # Example
+class Temporal(NamedObject):  # Example
     "Temporally attributed domain object."
 
     timepoint = RequiredFeature('Timepoint')
