@@ -11,12 +11,32 @@ from dm.dictionarywords import *
 
 # Todo: Fix this up more.
 
+class UmaskRotatingFileHandler(logging.handlers.RotatingFileHandler):
+
+    def _open(self):
+        dictionary = RequiredFeature('SystemDictionary')
+        umask = dictionary.getUmask()
+        prevumask = os.umask(umask)
+        rtv = logging.handlers.RotatingFileHandler._open(self)
+        os.umask(prevumask)
+        return rtv
+
+
 def initLogging():
     dictionary = RequiredFeature('SystemDictionary')
     logPath = os.path.abspath(dictionary[LOG_PATH])
     dirPath = os.path.dirname(logPath)
     if not os.path.exists(dirPath):
-        os.makedirs(dirPath)
+        # Need parents to have permission 750.
+        umask = os.umask(0o027)
+        try:
+            if not os.path.exists(os.path.dirname(dirPath)):
+                os.makedirs(os.path.dirname(dirPath))
+            # But need log directory to have permission 770.
+            os.umask(0o007)
+            os.makedirs(dirPath)
+        finally:
+            os.umask(umask)
     logLevel = logging.INFO
     logLevels = {
         'DEBUG'    : logging.DEBUG,
@@ -37,7 +57,8 @@ def initLogging():
     fileFormat = '[%(asctime)s] %(message)s'
     fileFormatter = logging.Formatter(fileFormat)
     
-    fileHandler = logging.handlers.RotatingFileHandler(
+    #fileHandler = logging.handlers.RotatingFileHandler(
+    fileHandler = UmaskRotatingFileHandler(
         logPath,
         mode='a+',
         maxBytes=10000000,
@@ -49,6 +70,7 @@ def initLogging():
     systemLogger = getLogger()
     systemLogger.addHandler(fileHandler)
     systemLogger.setLevel(logLevel)
+    LOG_PATH.test(logPath, dictionary)
     
 def getLogger():
     dictionary = RequiredFeature('SystemDictionary')
