@@ -7,6 +7,7 @@ from dm.strategy import BaseStrategy
 import os
 import sys
 from dm.on import json
+from dm.dom.pickers import GetMigrationAttributes
 
 # Todo: Support for merging data sets (id-fidelity is enforced now).
 # Todo: Add support for comments in plan files!
@@ -26,26 +27,29 @@ class DomainModelDumper(object):
         self.dictionary.set(DB_MIGRATION_IN_PROGRESS, '1')
         self.dataDump = {}
         domainClassRegister = self.registry.getDomainClassRegister()
+        # Dump all classes.
         for className in domainClassRegister.keys():
             classData = {}
             domainClass = domainClassRegister[className]
             #print "Dumping register %s" % className
-            # Dumping class meta data
+            # Dump class meta data.
+            picker = GetMigrationAttributes(domainClass.meta)
+            migrationAttrs = picker.pick()
             metaClassData = {}
-            for attr in domainClass.meta.attributes:
+            for attr in migrationAttrs:
                 metaClassData[attr.name] = attr.typeName
             classData['metaData'] = metaClassData
-            # Dumping class register objects
-            domainRegister = domainClass.createRegister()
-            if domainRegister.isStateful:
-                domainRegister = domainRegister.getAll()
-            for domainObject in domainRegister:
-                domainObjectData = domainObject.asDictValues()
+            # Dump all instances.
+            classRegister = domainClass.createRegister()
+            if classRegister.isStateful:
+                classRegister = classRegister.getAll()
+            for domainObject in classRegister:
+                manipulator = DomainObjectManipulator(classRegister, domainObject, pickerClass=GetMigrationAttributes)
+                domainObjectData = manipulator.domainObjectAsDictValues()
                 for (name, value) in domainObjectData.items():
                     if type(value) == unicode:
                         value = value.encode('utf8', 'ignore')
                         domainObjectData[name] = value
-                # Dumping object data
                 classData[domainObject.id] = domainObjectData
             self.dataDump[className] = classData
         self.dictionary.set(DB_MIGRATION_IN_PROGRESS, '')
@@ -64,16 +68,15 @@ class DomainModelDumper(object):
 
 class DomainModelLoader(object):
 
-    def __init__(self):
+    def __init__(self, isMerge=False):
+        self.isMerge = isMerge
         self.logger = RequiredFeature("Logger")
         self.registry = RequiredFeature("DomainRegistry")
         self.dictionary = RequiredFeature("SystemDictionary")
-        # Load plugin model classes into domain class register.
-        # Todo: Fix this, it doesn't bring the classes in.
-        self.pluginController = RequiredFeature("PluginController")
-        self.pluginController.getPlugins() 
         self.jsonDataDump = ''
         self.dataDump = None
+        if not self.isMerge and self.registry.states:
+            raise Exception, "Database is already initialised."
 
     def loadDataFromFile(self, dumpFilePath):
         # Todo: Wrap file access in try/except block.
@@ -114,6 +117,45 @@ class DomainModelLoader(object):
     def getDumpVersion(self):
         return self.dataDump['System']['1']['version']
 
+    def setDumpVersion(self, version):
+        self.dataDump['System']['1']['version'] = version
+
+    def deleteDomainClass(self, domainClassName):
+        if domainClassName in self.dataDump:
+            del(self.dataDump[domainClassName])
+
+    def findInstances(self, domainClassName, **kwds):
+        instances = {}
+        if domainClassName in self.dataDump:
+            for (id, data) in self.dataDump[domainClassName].items():
+                if id == 'metaData':
+                    continue
+                isMatch = True
+                for name, value in kwds.items():
+                    if data[name] != value:
+                        isMatch = False
+                        break
+                if isMatch:
+                    instances[id] = data
+        return instances
+
+    def addInstance(self, domainClassName, **data):
+        newId = self.getNewId(domainClassName)
+        self.dataDump[domainClassName][str(newId)] = data
+        return newId
+
+    def deleteInstance(self, domainClassName, oldId):
+        del(self.dataDump[domainClassName][str(oldId)])
+
+    def getNewId(self, domainClassName):
+        ids = self.getDomainClassIds(domainClassName)
+        return (len(ids) and ids[0] or 0) + 1
+
+    def getDomainClassIds(self, domainClassName):
+        ids = [int(i) for i in self.dataDump[domainClassName].keys() if i != 'metaData']
+        ids.sort(reverse=True)
+        return ids
+
     def identifyReferences(self, classNames):
         "Looks for HasA attrs on classes, collects by referenced class."
         self.unresolvedClassNames = []
@@ -179,13 +221,16 @@ class DomainModelLoader(object):
         # substitute ids where changed
         # store new id
         self.idMap = {}
+        #print "Import order:", self.importOrder
         for className in self.importOrder:
+            sys.stdout.write('\rImporting %s objects' % (className))
+            sys.stdout.flush()
             classData = self.dataDump[className]
             objectCount = len(classData.keys()) - 1
             domainClass = self.registry.getDomainClass(className)
             classRegister = domainClass.createRegister()
             #del(self.dataDump[className])
-            manipulator = DomainObjectManipulator(classRegister)
+            manipulator = DomainObjectManipulator(classRegister, pickerClass=GetMigrationAttributes)
             for objectId in classData.keys():
                 if objectId == 'metaData':
                     continue
@@ -220,7 +265,7 @@ class DomainModelLoader(object):
                         continue
                     elif objectData.has_key(attr.name):
                         value = objectData[attr.name]
-                        if attr.isDomainObjectRef and value.__class__ == int:
+                        if attr.isDomainObjectRef and value.__class__ in [int, long]:
                             idMapKey = "%s %s" % (attr.typeName, value)
                             if idMapKey in self.idMap.keys():
                                 mappedValue = self.idMap[idMapKey]
@@ -238,22 +283,31 @@ class DomainModelLoader(object):
                             strObjectData[attr.name] = value
                 if reflexiveStrObjectData:
                     reflexiveClassData[objectId] = reflexiveStrObjectData
-                objectDict = MultiValueDict()
+                objectDict = {} #MultiValueDict()
                 objectDict.update(strObjectData)
                 behindWithId = True
                 sacrificialRecords = []
                 try:
                     while(behindWithId):
-                        manipulator.create(objectDict)
+                        try:
+                            manipulator.create(objectDict)
+                        except:
+                            print objectDict
+                            raise
                         if manipulator.manipulationErrors:
-                            raise Exception("There were errors creating %s object: %s: %s" % (className, strObjectData, manipulator.manipulationErrors))
+                            msg = "There were errors creating %s object: %s: %s" % (className, strObjectData, manipulator.manipulationErrors)
+                            if self.dictionary[DB_MIGRATION_IN_PROGRESS]:
+                                print "Warning: %s" % msg
+                                break
+                            else:
+                                raise Exception(msg)
                         domainObject = manipulator.domainObject
                         if not domainObject:
                             raise Exception("Manipulator has no domain object.")
                         #print "Created %s" % domainObject
                         if domainObject.id > objectId:
-                            msg = "Index id %s is ahead of import id %s." % (
-                                domainObject.id, objectId
+                            msg = "Index id %s is ahead of import id %s (%s). Perhaps database already exists?" % (
+                                domainObject.id, objectId, className
                             )
                             #print "Warning: %s" % msg
                             raise Exception, msg
@@ -301,12 +355,11 @@ class DomainModelLoader(object):
             for id in reflexiveIds:
                 reflexiveStrObjectData = reflexiveClassData[id]
                 domainObject = freshDomainObjects.pop(id)
-                manipulator=DomainObjectManipulator(classRegister,domainObject)
-                objectDict = MultiValueDict()
+                objectDict = {}
                 objectDict.update(reflexiveStrObjectData)
                 for attrName in reflexiveStrObjectData:
-                    m = domainObject.meta.attributeNames[attrName]
-                    refObject = m.makeValueFromMultiValueDict(objectDict)
+                    metaAttr = domainObject.meta.attributeNames[attrName]
+                    refObject = metaAttr.makeValueFromMultiValueDict(objectDict)
                     setattr(domainObject, attrName, refObject)
                     domainObject.save()
 
