@@ -1,14 +1,21 @@
 from dm.ioc import *
 import dm.exceptions
 from dm.plugin.base import PluginBase
+from dm.dictionarywords import PLUGIN_PACKAGE_NAME
+from dm.dictionarywords import PLUGINS_AVAILABLE
+
+# Todo: Change the get methods to raise exceptions rather than returning None.
 
 class PluginFactory(object):
 
     dictionary = RequiredFeature('SystemDictionary')
     logger     = RequiredFeature('Logger')
 
+    def getAvailableNames(self):
+        return PLUGINS_AVAILABLE.parse(self.dictionary[PLUGINS_AVAILABLE])
+
     def getPlugin(self, domainObject):
-        "Builds plugin system from plugin domain object."
+        "Returns plugin system object for given plugin domain object."
         pluginName = domainObject.name
         pluginClass = self.getPluginClass(pluginName)
         if pluginClass:
@@ -17,6 +24,7 @@ class PluginFactory(object):
             return None
 
     def getPluginClass(self, pluginName):
+        "Returns plugin system class for given plugin name."
         pluginPackage = self.getPluginPackage(pluginName)
         pluginClass = None
         if pluginPackage:
@@ -39,7 +47,7 @@ class PluginFactory(object):
 
     def getPluginPackage(self, pluginName):
         "Imports named plugin package."
-        pluginPackageName = self.dictionary['plugin_package']
+        pluginPackageName = self.dictionary[PLUGIN_PACKAGE_NAME]
         pluginPackageName += '.' + pluginName
         try:
             pluginPackage = __import__(pluginPackageName, '', '', '*')
