@@ -15,6 +15,9 @@ class TestPluginFactory(TestCase):
 
     def setUp(self):
         super(TestPluginFactory, self).setUp()
+        self.setUpFactory()
+
+    def setUpFactory(self):
         self.factory = dm.plugin.factory.PluginFactory()
 
     def pluginName(self):
@@ -27,11 +30,17 @@ class TestPluginFactory(TestCase):
     def testExists(self):
         self.failUnless(self.factory, "No factory was created.")
 
-    def testGetPlugin(self):
+    def test_getAvailableNames(self):
+        self.failUnless(self.pluginName() in self.factory.getAvailableNames())
+
+    def test_getPlugin(self):
         plugin = self.factory.getPlugin(self.pluginDomainObject())
         self.failUnless(plugin, "No plugin produced by factory.")
         self.failUnlessEqual(plugin.domainObject, self.pluginDomainObject())
 
+    def test_getPluginClass(self):
+        self.failUnless(self.factory.getPluginClass(self.pluginName()))
+
     def testOnRun(self):
         plugin = self.factory.getPlugin(self.pluginDomainObject())
         countBefore = plugin.counts['onRun']
@@ -39,24 +48,3 @@ class TestPluginFactory(TestCase):
         countAfter = plugin.counts['onRun']
         self.failUnless(countBefore + 1 == countAfter)
 
-    def testOnProjectCreate(self):
-        plugin = self.factory.getPlugin(self.pluginDomainObject())
-        countBefore = plugin.counts['onProjectCreate']
-        val = plugin.onProjectCreate(None)
-        countAfter = plugin.counts['onProjectCreate']
-        self.failUnless(countBefore + 1 == countAfter)
-
-    def testOnProjectApprove(self):
-        plugin = self.factory.getPlugin(self.pluginDomainObject())
-        countBefore = plugin.counts['onProjectApprove']
-        val = plugin.onProjectApprove(None)
-        countAfter = plugin.counts['onProjectApprove']
-        self.failUnless(countBefore + 1 == countAfter)
-
-    def testOnProjectDelete(self):
-        plugin = self.factory.getPlugin(self.pluginDomainObject())
-        countBefore = plugin.counts['onProjectDelete']
-        val = plugin.onProjectDelete(None)
-        countAfter = plugin.counts['onProjectDelete']
-        self.failUnless(countBefore + 1 == countAfter)
-
