@@ -1,6 +1,7 @@
 from dm.testunit import *
 import dm.plugin.basetest
 import dm.plugin.controllertest
+import dm.plugin.factorytest
 import unittest
 import os
 
@@ -9,6 +10,7 @@ def suite():
     suites = [
         dm.plugin.basetest.suite(),
         dm.plugin.controllertest.suite(),
+        dm.plugin.factorytest.suite(),
     ]
     return unittest.TestSuite(suites)
 
