@@ -5,6 +5,7 @@ from dm.exceptions import KforgeRegistryKeyError
 from dm.messagedigest import md5
 from dm.dom.base import DomainObject, DomainObjectRegister
 moddebug = False
+import string
 
 class BaseStrategy(object):
 
@@ -27,6 +28,11 @@ class EpochFromDateTime(BaseStrategy):
         return int(self.dateTime)
 
 
+# Todo: Split up making protected names, to be more explicit about whether the names
+# are for the instance, the class, the register, or just a name. Need to remove
+# protection objects for instances without risking removing protection object for class
+# and this code doesn't quite eliminate that risk. It's not broken though, but still... 
+
 class MakeProtectedNames(BaseStrategy):
 
     def __init__(self, protectedObject):
@@ -53,6 +59,8 @@ class MakeProtectedNames(BaseStrategy):
             instanceName = className + "." + str(keyValue)
             protectedNames.append(instanceName)
             protectedNames.append(className)
+        elif isinstance(self.protectedObject, (str, unicode)):
+            protectedNames.append(self.protectedObject)
         else:
             protectedNames.append('System')
         if moddebug and self.debug:
@@ -94,9 +102,13 @@ class FindProtectionObjects(BaseStrategy):
 
 
 class FindInstanceProtectionObject(FindProtectionObjects):
+    """Returns protection object for the domain object instance (and not for
+    the class as a whole)."""
 
     def find(self):
         protectionObjects = super(FindInstanceProtectionObject, self).find()
+        # Todo: Make sure the protection object has no dots? Or search for
+        # protection objects that match the protected name of the instance?
         if len(protectionObjects) == 2:
             return protectionObjects[0]
         else:
