@@ -21,6 +21,7 @@ import dm.viewtest
 import dm.djangotest
 import dm.pylonstest
 import dm.datetimeconvertortest
+import dm.utiltest
 from dm.testunit import ApplicationTestSuite
 
 def suite():
@@ -45,6 +46,7 @@ def suite():
         dm.migratetest.suite(),
         dm.djangotest.suite(),
         dm.pylonstest.suite(),
+        dm.utiltest.suite(),
         #dm.filesystemtest.suite(),
         #dm.applicationtest.suite(),
     ]
