@@ -1,17 +1,28 @@
+from dm.dom.pickers import GetAdminInitableAttributes
+from dm.dom.pickers import GetAdminReadableAttributes
+from dm.dom.pickers import GetAdminEditableAttributes
 from dm.view.base import *
 
 class AdminView(SessionView):
     
     minorNavigation = [
-        {'title': 'Admin',      'url': '/admin/'},
         {'title': 'Model', 'url': '/admin/model/'},
     ]
-    majorNavigationItem = '/'
+    majorNavigationItem = '/admin/'
     minorNavigationItem = '/admin/'
 
     def canAccess(self):
         return self.canUpdateSystem()
 
+    def getCreatePickerClass(self):
+        return GetAdminInitableAttributes
+    
+    def getReadPickerClass(self):
+        return GetAdminReadableAttributes
+    
+    def getUpdatePickerClass(self):
+        return GetAdminEditableAttributes
+    
 
 class AdminIndexView(AdminView):
 
@@ -39,26 +50,41 @@ class AdminListView(AbstractListView, AdminView):
     minorNavigationItem = '/admin/model/'
         
 
-class AdminCreateView(AbstractCreateView, AdminView):
+class AdminCreateView(AdminView, AbstractCreateView):
 
     templatePath = 'admin/create'
     minorNavigationItem = '/admin/model/'
-    
+
+    def getPickerClass(self):
+        return GetAdminInitableAttributes
+
     def makePostManipulateLocation(self):
         return '/admin/model/%s/' % self.domainClassName
 
 
-class AdminReadView(AbstractReadView, AdminView):
+class AdminReadView(AdminView, AbstractReadView):
 
     templatePath = 'admin/read'
     minorNavigationItem = '/admin/model/'
 
+    def getPickerClass(self):
+        return GetAdminReadableAttributes
 
-class AdminUpdateView(AbstractUpdateView, AdminView):
+    def setContext(self):
+        super(AdminReadView, self).setContext()
+        self.context.update({
+            'domainObjectNamedValues': self.getDomainObject().asNamedValues(),
+        })
+
+        
+class AdminUpdateView(AdminView, AbstractUpdateView):
 
     templatePath = 'admin/update'
     minorNavigationItem = '/admin/model/'
     
+    def getPickerClass(self):
+        return GetAdminEditableAttributes
+
     def makePostManipulateLocation(self):
         return '/admin/model/%s/%s/' % (
             self.domainClassName, 
@@ -66,50 +92,68 @@ class AdminUpdateView(AbstractUpdateView, AdminView):
         )
 
 
-class AdminDeleteView(AbstractDeleteView, AdminView):
+class AdminDeleteView(AdminView, AbstractDeleteView):
 
     templatePath = 'admin/delete'
     minorNavigationItem = '/admin/model/'
 
+    def getPickerClass(self):
+        return GetAdminReadableAttributes
+
     def makePostManipulateLocation(self):
         return '/admin/model/%s/' % self.domainClassName
 
 
-class AdminListHasManyView(AbstractListHasManyView, AdminView):
+class AdminListHasManyView(AdminView, AbstractListHasManyView):
 
     templatePath = 'admin/listHasMany'
     minorNavigationItem = '/admin/model/'
 
 
-class AdminCreateHasManyView(AbstractCreateHasManyView, AdminView):
+class AdminCreateHasManyView(AdminView, AbstractCreateHasManyView):
 
     templatePath = 'admin/createHasMany'
     minorNavigationItem = '/admin/model/'
 
+    def getPickerClass(self):
+        return GetAdminInitableAttributes
+
     def makePostManipulateLocation(self):
         return '/admin/model/%s/%s/%s/' % (
             self.domainClassName, self.domainObjectKey, self.hasManyName
         )
 
 
-class AdminReadHasManyView(AbstractReadHasManyView, AdminView):
+class AdminReadHasManyView(AdminView, AbstractReadHasManyView):
 
     templatePath = 'admin/readHasMany'
     minorNavigationItem = '/admin/model/'
 
+    def getPickerClass(self):
+        return GetAdminReadableAttributes
 
-class AdminUpdateHasManyView(AbstractUpdateHasManyView, AdminView):
+
+class AdminUpdateHasManyView(AdminView, AbstractUpdateHasManyView):
 
     templatePath = 'admin/updateHasMany'
     minorNavigationItem = '/admin/model/'
 
     def makePostManipulateLocation(self):
-        return '/admin/model/%s/%s/%s/' % (
-            self.domainClassName, self.domainObjectKey, self.hasManyName
+        register = self.getHasManyRegister()
+        manipulatedObject = self.getManipulatedDomainObject()
+        hasManyKey = register.getRegisterKey(manipulatedObject)
+        if register.getKeyMeta() and register.getKeyMeta().isDomainObjectRef:
+            hasManyKey = hasManyKey.getRegisterKeyValue()
+        return '/admin/model/%s/%s/%s/%s/' % (
+            #self.domainClassName, self.domainObjectKey, self.hasManyName, self.hasManyKey
+            self.domainClassName, self.domainObjectKey, self.hasManyName, hasManyKey
         )
 
+    def getPickerClass(self):
+        return GetAdminEditableAttributes
 
-class AdminDeleteHasManyView(AbstractDeleteHasManyView, AdminView):
+
+class AdminDeleteHasManyView(AdminView, AbstractDeleteHasManyView):
 
     templatePath = 'admin/deleteHasMany'
     minorNavigationItem = '/admin/model/'
@@ -119,6 +163,10 @@ class AdminDeleteHasManyView(AbstractDeleteHasManyView, AdminView):
             self.domainClassName, self.domainObjectKey, self.hasManyName
         )
 
+    def getPickerClass(self):
+        return GetAdminReadableAttributes
+
+
 
 def index(request):
     view = AdminIndexView(request=request)
