@@ -156,7 +156,7 @@ class TestCreateView(AdminClassTestCase):
         request.POST['name']     = self.fixtureKeyName
         request.POST['fullname'] = self.fixtureFullname
         request.POST['password'] = self.fixtureKeyName
-        request.POST['email']    = self.fixtureKeyName +"@"+ self.fixtureKeyName +".com"
+        #request.POST['email']    = self.fixtureKeyName +"@"+ self.fixtureKeyName +".com"
         request.POST['role']     = self.fixtureSystemRole
         request.POST['state']    = self.fixtureState
         return request
@@ -220,7 +220,7 @@ class TestUpdateView(AdminInstanceTestCase):
         request.POST['name']     = self.fixtureKeyName
         request.POST['fullname'] = self.fixtureFullname + " Updated"
         request.POST['password'] = self.fixtureKeyName
-        request.POST['email']    = self.fixtureKeyName
+        #request.POST['email']    = self.fixtureKeyName
         request.POST['role']     = self.fixtureSystemRole
         request.POST['state']    = self.fixtureState
         return request
