@@ -114,14 +114,14 @@ class TestRegistryAutocompleterView(AutocompleterViewTestCase):
 
     def setUp(self):
         sessions = self.registry.sessions
-        persons = self.registry.persons
-        self.person = persons.create('TestRegistryAutocomplete')
+        people = self.registry.people
+        self.person = people.create('TestRegistryAutocomplete')
         self.session1 = sessions.create('RegistryAutocomplete1')
         self.session2 = sessions.create('RegistryAutocomplete2')
         self.requiredResponseContent = '["RegistryAutocomplete2 (#%s)"]' % (
             self.session2.id
         )
-        self.registryPath = 'persons/%s/sessions' % self.person.getRegisterKeyValue()
+        self.registryPath = 'people/%s/sessions' % self.person.getRegisterKeyValue()
         super(TestRegistryAutocompleterView, self).setUp()
 
     def tearDown(self):
@@ -145,8 +145,8 @@ class AutoappenderViewTestCase(JsonViewTestCase):
         self.POST['targetAttribute'] = self.targetAttribute
 
     def setUp(self):
-        while 'TestRegistryAutoappend' in self.registry.persons:
-            del(self.registry.persons['TestRegistryAutoappend'])
+        while 'TestRegistryAutoappend' in self.registry.people:
+            del(self.registry.people['TestRegistryAutoappend'])
         while 'RegistryAutoappend1' in self.registry.sessions:
             del(self.registry.sessions['RegistryAutoappend1'])
         while 'RegistryAutoappend2' in self.registry.sessions:
@@ -154,12 +154,12 @@ class AutoappenderViewTestCase(JsonViewTestCase):
         while 'RegistryAutoappend3' in self.registry.sessions:
             del(self.registry.sessions['RegistryAutoappend3'])
         sessions = self.registry.sessions
-        persons = self.registry.persons
-        self.person = persons.create('TestRegistryAutoappend')
+        people = self.registry.people
+        self.person = people.create('TestRegistryAutoappend')
         self.session1 = sessions.create('RegistryAutoappend1')
         self.session2 = sessions.create('RegistryAutoappend2')
         registerKey = self.person.getRegisterKeyValue()
-        self.registryPath = 'persons/%s/sessions' % registerKey 
+        self.registryPath = 'people/%s/sessions' % registerKey 
         super(AutoappenderViewTestCase, self).setUp()
 
     def tearDown(self):
@@ -170,8 +170,8 @@ class AutoappenderViewTestCase(JsonViewTestCase):
             del(self.registry.sessions['RegistryAutoappend2'])
         while 'RegistryAutoappend1' in self.registry.sessions:
             del(self.registry.sessions['RegistryAutoappend1'])
-        while 'TestRegistryAutoappend' in self.registry.persons:
-            del(self.registry.persons['TestRegistryAutoappend'])
+        while 'TestRegistryAutoappend' in self.registry.people:
+            del(self.registry.people['TestRegistryAutoappend'])
 
     def checkResponseContent(self):
         session = self.registry.sessions[self.completedString]
@@ -195,21 +195,21 @@ class AutodeleteViewTestCase(JsonViewTestCase):
         self.POST['registryPath'] = self.registryPath
 
     def setUp(self):
-        while 'TestRegistryAutoappend' in self.registry.persons:
-            del(self.registry.persons['TestRegistryAutoappend'])
+        while 'TestRegistryAutoappend' in self.registry.people:
+            del(self.registry.people['TestRegistryAutoappend'])
         while 'RegistryAutoappend1' in self.registry.sessions:
             del(self.registry.sessions['RegistryAutoappend1'])
         while 'RegistryAutoappend2' in self.registry.sessions:
             del(self.registry.sessions['RegistryAutoappend2'])
         while 'RegistryAutoappend3' in self.registry.sessions:
             del(self.registry.sessions['RegistryAutoappend3'])
-        persons = self.registry.persons
-        self.person = persons.create('TestRegistryAutoappend')
+        people = self.registry.people
+        self.person = people.create('TestRegistryAutoappend')
         sessions = self.person.sessions
         self.session1 = sessions.create('RegistryAutoappend1')
         self.session2 = sessions.create('RegistryAutoappend2')
         registerKey = self.person.getRegisterKeyValue()
-        self.registryPath = 'persons/%s/sessions/%s' % (
+        self.registryPath = 'people/%s/sessions/%s' % (
             registerKey, 'RegistryAutoappend1')
         super(AutodeleteViewTestCase, self).setUp()
 
@@ -221,8 +221,8 @@ class AutodeleteViewTestCase(JsonViewTestCase):
             del(self.registry.sessions['RegistryAutoappend2'])
         while 'RegistryAutoappend1' in self.registry.sessions:
             del(self.registry.sessions['RegistryAutoappend1'])
-        while 'TestRegistryAutoappend' in self.registry.persons:
-            del(self.registry.persons['TestRegistryAutoappend'])
+        while 'TestRegistryAutoappend' in self.registry.people:
+            del(self.registry.people['TestRegistryAutoappend'])
 
     def checkResponseContent(self):
         self.failIf('RegistryAutoappend1' in self.person.sessions, self.person.sessions.keys()) 
