@@ -4,6 +4,7 @@ import dm.view.basetest
 import dm.view.admintest
 import dm.view.registrytest
 import dm.view.rpctest
+import dm.view.apitest
 
 def suite():
     suites = [
@@ -12,6 +13,7 @@ def suite():
         dm.view.admintest.suite(),
         dm.view.registrytest.suite(),
         dm.view.rpctest.suite(),
+        dm.view.apitest.suite(),
     ]
     return unittest.TestSuite(suites)
 
